@@ -5,20 +5,20 @@ Revises: 070a0eb89e88
 Create Date: 2020-06-09 21:31:07.912724
 
 """
+
 from alembic import op
 import sqlalchemy as sa
 
-
 # revision identifiers, used by Alembic.
-revision = '9da58d2def22'
-down_revision = '070a0eb89e88'
+revision = "9da58d2def22"
+down_revision = "070a0eb89e88"
 branch_labels = None
 depends_on = None
 
 
 def upgrade():
-    m = sa.MetaData(op.get_bind())
-    m.reflect()
+    m = sa.MetaData()
+    m.reflect(op.get_bind())
     if "next_id" in m.tables:
         with op.batch_alter_table("next_id") as batch_op:
             batch_op.add_column(sa.Column("entity_group_id", sa.Integer))
