@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -10,10 +11,7 @@
 ######################################################################################################################
 
 """
-Classes to handle exceptions while using the Spine database API.
-
-:author: Manuel Marin (KTH)
-:date:   15.8.2018
+Spine DB API exceptions.
 """
 
 
@@ -36,49 +34,22 @@ class SpineIntegrityError(SpineDBAPIError):
         id (int): the id the instance that caused a unique violation
     """
 
-    def __init__(self, msg=None, id=None):
+    def __init__(self, msg=None, id_=None):
         super().__init__(msg)
-        self.id = id
+        self.id = id_
 
 
 class SpineDBVersionError(SpineDBAPIError):
     """Database version error."""
 
     def __init__(self, url=None, current=None, expected=None, upgrade_available=True):
-        super().__init__(msg="The database at '{}' is not the expected version.".format(url))
+        super().__init__(msg=f"The database at '{url}' is not the expected version.")
         self.url = url
         self.current = current
         self.expected = expected
         self.upgrade_available = upgrade_available
 
 
-class SpineTableNotFoundError(SpineDBAPIError):
-    """Can't find one of the tables."""
-
-    def __init__(self, table, url=None):
-        super().__init__(msg="Table(s) '{}' couldn't be mapped from the database at '{}'.".format(table, url))
-        self.table = table
-
-
-class RecordNotFoundError(SpineDBAPIError):
-    """Can't find one record in one of the tables."""
-
-    def __init__(self, table, name=None, id=None):
-        super().__init__(msg="Unable to find item in table '{}'.".format(table))
-        self.table = table
-        self.name = name
-        self.id = id
-
-
-class ParameterValueError(SpineDBAPIError):
-    """The value given for a parameter does not fit the datatype."""
-
-    def __init__(self, value, data_type):
-        super().__init__(msg="The value {} does not fit the datatype '{}'.".format(value, data_type))
-        self.value = value
-        self.data_type = data_type
-
-
 class ParameterValueFormatError(SpineDBAPIError):
     """
     Failure in encoding/decoding a parameter value.
@@ -93,7 +64,7 @@ class ParameterValueFormatError(SpineDBAPIError):
 
 class InvalidMapping(SpineDBAPIError):
     """
-    Failure in import/export mapping
+    Failure in import/export mapping.
     """
 
     def __init__(self, msg):
@@ -105,3 +76,21 @@ class InvalidMappingComponent(InvalidMapping):
         super().__init__(msg)
         self.rank = rank
         self.key = key
+
+
+class ReaderError(SpineDBAPIError):
+    """Failure in import reader."""
+
+
+class NothingToCommit(SpineDBAPIError):
+    """A notification that the session contains no changes."""
+
+    def __init__(self):
+        super().__init__("Nothing to commit.")
+
+
+class NothingToRollback(SpineDBAPIError):
+    """A notification that the session contains no changes."""
+
+    def __init__(self):
+        super().__init__("Nothing to rollback.")
