@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -8,30 +9,17 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""
-This package contains facilities to map a Spine database into tables.
-
-:author: A. Soininen (VTT)
-:date:   10.12.2020
-"""
 
 from .generator import rows, titles
 from .settings import (
     alternative_export,
-    feature_export,
-    object_export,
-    object_group_export,
-    object_parameter_default_value_export,
-    object_parameter_export,
+    entity_dimension_parameter_default_value_export,
+    entity_dimension_parameter_value_export,
+    entity_export,
+    entity_group_export,
+    entity_parameter_default_value_export,
+    entity_parameter_value_export,
     parameter_value_list_export,
-    relationship_export,
-    relationship_object_parameter_default_value_export,
-    relationship_object_parameter_export,
-    relationship_parameter_default_value_export,
-    relationship_parameter_export,
     scenario_alternative_export,
     scenario_export,
-    tool_export,
-    tool_feature_export,
-    tool_feature_method_export,
 )
