@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -8,24 +9,31 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-
-"""
-Tools and utilities to work with filters, manipulators and database URLs.
-
-:author: A. Soininen
-:date:   7.12.2020
-"""
+""" This module contains tools and utilities to work with filters. """
+from __future__ import annotations
+from collections.abc import Iterable
+from itertools import dropwhile, takewhile
 from json import dump, load
-from urllib.parse import parse_qs, urlencode, urlparse, urlunparse
+import sys
+from typing import Optional, Union
+from urllib.parse import parse_qs, urlencode, urlparse
 from .alternative_filter import (
     ALTERNATIVE_FILTER_SHORTHAND_TAG,
     ALTERNATIVE_FILTER_TYPE,
     alternative_filter_config,
-    alternative_filter_from_dict,
     alternative_filter_config_to_shorthand,
+    alternative_filter_from_dict,
     alternative_filter_shorthand_to_config,
     alternative_names_from_dict,
 )
+from .execution_filter import (
+    EXECUTION_FILTER_TYPE,
+    EXECUTION_SHORTHAND_TAG,
+    execution_filter_config,
+    execution_filter_config_to_shorthand,
+    execution_filter_from_dict,
+    execution_filter_shorthand_to_config,
+)
 from .renamer import (
     ENTITY_CLASS_RENAMER_SHORTHAND_TAG,
     ENTITY_CLASS_RENAMER_TYPE,
@@ -47,54 +55,33 @@ from .scenario_filter import (
     scenario_filter_shorthand_to_config,
     scenario_name_from_dict,
 )
-from .tool_filter import (
-    TOOL_SHORTHAND_TAG,
-    TOOL_FILTER_TYPE,
-    tool_filter_config,
-    tool_filter_config_to_shorthand,
-    tool_filter_from_dict,
-    tool_filter_shorthand_to_config,
-    tool_name_from_dict,
-)
 from .value_transformer import (
     VALUE_TRANSFORMER_SHORTHAND_TAG,
     VALUE_TRANSFORMER_TYPE,
-    value_transformer_shorthand_to_config,
-    value_transformer_from_dict,
     value_transformer_config_to_shorthand,
+    value_transformer_from_dict,
+    value_transformer_shorthand_to_config,
 )
-from .execution_filter import (
-    EXECUTION_SHORTHAND_TAG,
-    EXECUTION_FILTER_TYPE,
-    execution_filter_config,
-    execution_filter_config_to_shorthand,
-    execution_filter_from_dict,
-    execution_filter_shorthand_to_config,
-)
+
+__all__ = ("append_filter_config", "apply_filter_stack", "clear_filter_configs", "filter_configs", "name_from_dict")
 
 FILTER_IDENTIFIER = "spinedbfilter"
 SHORTHAND_TAG = "cfg:"
 
 
-def apply_filter_stack(db_map, stack):
-    """
-    Applies stack of filters and manipulator to given database map.
-
-    Args:
-        db_map (DatabaseMappingBase): a database map
-        stack (list): a stack of database filters and manipulators
-    """
+def apply_filter_stack(db_map: DatabaseMapping, stack: Iterable[dict]) -> None:
+    """Applies filters given as config dicts in ``stack`` to a database map."""
     appliers = {
         ALTERNATIVE_FILTER_TYPE: alternative_filter_from_dict,
         ENTITY_CLASS_RENAMER_TYPE: entity_class_renamer_from_dict,
         EXECUTION_FILTER_TYPE: execution_filter_from_dict,
         PARAMETER_RENAMER_TYPE: parameter_renamer_from_dict,
         SCENARIO_FILTER_TYPE: scenario_filter_from_dict,
-        TOOL_FILTER_TYPE: tool_filter_from_dict,
         VALUE_TRANSFORMER_TYPE: value_transformer_from_dict,
     }
-    for filter_ in stack:
-        appliers[filter_["type"]](db_map, filter_)
+    with db_map:
+        for filter_ in stack:
+            appliers[filter_["type"]](db_map, filter_)
 
 
 def load_filters(configs):
@@ -107,10 +94,10 @@ def load_filters(configs):
     Returns:
         list of dict: filter stack
     """
-    stack = list()
+    stack = []
     for config in configs:
         if isinstance(config, str):
-            with open(config) as config_file:
+            with open(config, encoding="utf-8") as config_file:
                 stack.append(load(config_file))
         else:
             stack.append(config)
@@ -141,55 +128,37 @@ def filter_config(filter_type, value):
     """
     return {
         SCENARIO_FILTER_TYPE: scenario_filter_config,
-        TOOL_FILTER_TYPE: tool_filter_config,
         ALTERNATIVE_FILTER_TYPE: alternative_filter_config,
         EXECUTION_FILTER_TYPE: execution_filter_config,
     }[filter_type](value)
 
 
-def append_filter_config(url, config):
+def append_filter_config(url: str, config: Union[dict, str]) -> str:
     """
     Appends a filter config to given url.
 
-    ``config`` can either be a configuration dictionary or a path to a JSON file that contains the dictionary.
-
-    Args:
-        url (str): base URL
-        config (str or dict): path to the config file or config as a ``dict``.
-
-    Returns:
-        str: the modified URL
+    ``config`` can either be a config dict, a shorthand, or a path to a JSON file that contains the dictionary.
     """
     url = urlparse(url)
     query = parse_qs(url.query)
-    filters = query.setdefault(FILTER_IDENTIFIER, list())
+    filters = query.setdefault(FILTER_IDENTIFIER, [])
     if isinstance(config, dict):
         config = config_to_shorthand(config)
     if config not in filters:
         filters.append(config)
     url = url._replace(query=urlencode(query, doseq=True))
-    if not url.hostname:
-        url = url._replace(path="//" + url.path)
-    return url.geturl()
+    return _unparse_url_ensuring_correct_slashes(url)
 
 
-def filter_configs(url):
-    """
-    Returns filter configs or file paths from given URL.
-
-    Args:
-        url (str): a URL
-
-    Returns:
-        list: a list of filter configs
-    """
+def filter_configs(url: str) -> list[dict]:
+    """Returns filter config dicts or file paths to config JSON from given URL."""
     parsed = urlparse(url)
     query = parse_qs(parsed.query)
     try:
         filters = query[FILTER_IDENTIFIER]
     except KeyError:
         return []
-    parsed_filters = list()
+    parsed_filters = []
     for filter_ in filters:
         if filter_.startswith(SHORTHAND_TAG):
             parsed_filters.append(_parse_shorthand(filter_[len(SHORTHAND_TAG) :]))
@@ -214,36 +183,26 @@ def pop_filter_configs(url):
         filters = query.pop(FILTER_IDENTIFIER)
     except KeyError:
         return [], url
-    parsed_filters = list()
+    parsed_filters = []
     for filter_ in filters:
         if filter_.startswith(SHORTHAND_TAG):
             parsed_filters.append(_parse_shorthand(filter_[len(SHORTHAND_TAG) :]))
         else:
             parsed_filters.append(filter_)
     parsed = parsed._replace(query=urlencode(query, doseq=True))
-    if not parsed.hostname:
-        parsed = parsed._replace(path="//" + parsed.path)
-    return parsed_filters, urlunparse(parsed)
+    return parsed_filters, _unparse_url_ensuring_correct_slashes(parsed)
 
 
-def clear_filter_configs(url):
-    """
-    Removes filter configuration queries from given URL.
-
-    Args:
-        url (str): a URL
-
-    Returns:
-        str: a cleared URL
-    """
+def clear_filter_configs(url: str) -> str:
+    """Removes filters from given URL returning the cleared URL."""
     parsed = urlparse(url)
     query = parse_qs(parsed.query)
     try:
         del query[FILTER_IDENTIFIER]
     except KeyError:
         return url
-    parsed = parsed._replace(query=urlencode(query, doseq=True), path="//" + parsed.path)
-    return urlunparse(parsed)
+    parsed = parsed._replace(query=urlencode(query, doseq=True))
+    return _unparse_url_ensuring_correct_slashes(parsed)
 
 
 def ensure_filtering(url, fallback_alternative=None):
@@ -290,7 +249,6 @@ def config_to_shorthand(config):
         ENTITY_CLASS_RENAMER_TYPE: entity_class_renamer_config_to_shorthand,
         PARAMETER_RENAMER_TYPE: parameter_renamer_config_to_shorthand,
         SCENARIO_FILTER_TYPE: scenario_filter_config_to_shorthand,
-        TOOL_FILTER_TYPE: tool_filter_config_to_shorthand,
         EXECUTION_FILTER_TYPE: execution_filter_config_to_shorthand,
         VALUE_TRANSFORMER_TYPE: value_transformer_config_to_shorthand,
     }
@@ -312,7 +270,6 @@ def _parse_shorthand(shorthand):
         ENTITY_CLASS_RENAMER_SHORTHAND_TAG: entity_class_renamer_shorthand_to_config,
         PARAMETER_RENAMER_SHORTHAND_TAG: parameter_renamer_shorthand_to_config,
         SCENARIO_SHORTHAND_TAG: scenario_filter_shorthand_to_config,
-        TOOL_SHORTHAND_TAG: tool_filter_shorthand_to_config,
         EXECUTION_SHORTHAND_TAG: execution_filter_shorthand_to_config,
         VALUE_TRANSFORMER_SHORTHAND_TAG: value_transformer_shorthand_to_config,
     }
@@ -320,17 +277,28 @@ def _parse_shorthand(shorthand):
     return shorthand_parsers[tag](shorthand)
 
 
-def name_from_dict(config):
-    """
-    Returns scenario or tool name from filter config.
-
-    Args:
-        config (dict): filter configuration
-
-    Returns:
-        str: name or None if ``config`` is not a valid 'name' filter configuration
-    """
-    func = {SCENARIO_FILTER_TYPE: scenario_name_from_dict, TOOL_FILTER_TYPE: tool_name_from_dict}.get(config["type"])
+def name_from_dict(config: dict) -> Optional[str]:
+    """Returns scenario name from scenario filter config dict or None for other filter types."""
+    func = {SCENARIO_FILTER_TYPE: scenario_name_from_dict}.get(config["type"])
     if func is None:
         return None
     return func(config)
+
+
+def _unparse_url_ensuring_correct_slashes(url):
+    """Converts URL tuple into string ensuring SqlAlchemy compatible format for SQLite URLs.
+
+    Args:
+        url (NamedTuple): URL tuple
+
+    Returns:
+        str: URL as string
+    """
+    url_string = url.geturl()
+    if not url.hostname:
+        slash_count = len(list(takewhile(lambda c: c == "/", dropwhile(lambda c: c != "/", url_string))))
+        if slash_count != 3:
+            scheme, _, rest = url_string.partition(slash_count * "/" or "//")
+            slashes = ("///" if sys.platform == "win32" or "\\" in url.path else "////") if url.path else "//"
+            return scheme + slashes + rest
+    return url_string
