@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -8,35 +9,37 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
+""" General helper functions. """
 
-"""
-General helper functions and classes.
-
-:author: Manuel Marin (KTH)
-:date:   15.8.2018
-"""
-
-import os
+from collections.abc import Callable, Iterable, Iterator, Sequence
+import enum
+from itertools import groupby
 import json
-import warnings
 from operator import itemgetter
-from urllib.parse import urlparse, urlunparse
+import os
+from typing import Any
+import warnings
+from alembic.config import Config
+from alembic.environment import EnvironmentContext
+from alembic.migration import MigrationContext
+from alembic.script import ScriptDirectory
 from sqlalchemy import (
-    Boolean,
     BigInteger,
+    Boolean,
     CheckConstraint,
     Column,
     DateTime,
+    Enum,
     Float,
     ForeignKey,
     ForeignKeyConstraint,
     Integer,
+    LargeBinary,
     MetaData,
     PrimaryKeyConstraint,
     String,
     Table,
     Text,
-    LargeBinary,
     UniqueConstraint,
     create_engine,
     false,
@@ -44,28 +47,30 @@ from sqlalchemy import (
     inspect,
     null,
     select,
+    text,
+    true,
 )
-from sqlalchemy.ext.automap import generate_relationship
+from sqlalchemy.dialects.mysql import DOUBLE, TINYINT
+from sqlalchemy.engine import Engine
+from sqlalchemy.exc import DatabaseError, IntegrityError
 from sqlalchemy.ext.compiler import compiles
-from sqlalchemy.exc import DatabaseError, IntegrityError, OperationalError
-from sqlalchemy.dialects.mysql import TINYINT, DOUBLE
 from sqlalchemy.sql.expression import FunctionElement, bindparam, cast
-from alembic.config import Config
-from alembic.script import ScriptDirectory
-from alembic.migration import MigrationContext
-from alembic.environment import EnvironmentContext
+from sqlalchemy.sql.selectable import SelectBase
 from .exception import SpineDBAPIError, SpineDBVersionError
 
-# Supported dialects and recommended dbapi. Restricted to mysql and sqlite for now:
-# - sqlite works
-# - mysql is trying to work
 SUPPORTED_DIALECTS = {
     "mysql": "pymysql",
     "sqlite": "sqlite3",
-    # "mssql": "pyodbc",
-    # "postgresql": "psycopg2",
-    # "oracle": "cx_oracle",
 }
+"""Currently supported dialects and recommended dbapi."""
+
+
+UNSUPPORTED_DIALECTS = {
+    "mssql": "pyodbc",
+    "postgresql": "psycopg2",
+}
+"""Dialects and recommended dbapi that are not supported by DatabaseMapping but are supported by SqlAlchemy."""
+
 
 naming_convention = {
     "pk": "pk_%(table_name)s",
@@ -76,19 +81,19 @@ naming_convention = {
 
 model_meta = MetaData(naming_convention=naming_convention)
 
-LONGTEXT_LENGTH = 2 ** 32 - 1
+LONGTEXT_LENGTH = 2**32 - 1
 
-# NOTE: Deactivated since foreign keys are too difficult to get right in the diff tables.
-# For example, the diff_object table would need a `class_id` field and a `diff_class_id` field,
-# plus a CHECK constraint that at least one of the two is NOT NULL.
-# @event.listens_for(Engine, "connect")
-def set_sqlite_pragma(dbapi_connection, connection_record):
-    module_name = dbapi_connection.__class__.__module__
-    if not module_name.lower().startswith("sqlite"):
-        return
-    cursor = dbapi_connection.cursor()
-    cursor.execute("PRAGMA foreign_keys=ON")
-    cursor.close()
+
+def name_from_elements(elements: Sequence[str]) -> str:
+    """Creates an entity name by combining a list of element names into a single string."""
+    if len(elements) == 1:
+        return elements[0] + "__"
+    return "__".join(elements)
+
+
+def name_from_dimensions(dimensions: Sequence[str]) -> str:
+    """Creates an entity class name by combining a list of dimension names into a single string."""
+    return name_from_elements(dimensions)
 
 
 @compiles(TINYINT, "sqlite")
@@ -105,7 +110,8 @@ def compile_DOUBLE_mysql_sqlite(element, compiler, **kw):
 
 class group_concat(FunctionElement):
     type = String()
-    name = 'group_concat'
+    name = "group_concat"
+    inherit_cache = True
 
 
 def _parse_group_concat_clauses(clauses):
@@ -126,17 +132,17 @@ def compile_group_concat_mysql(element, compiler, **kw):
     str_group_concat_column = cast(group_concat_column, String)
     if order_by_column is not None:
         str_group_concat_column = str_group_concat_column.op("ORDER BY")(order_by_column)
-    return "group_concat(%s separator %s)" % (
-        compiler.process(str_group_concat_column, **kw),
-        compiler.process(separator, **kw),
+    return (
+        f"group_concat({compiler.process(str_group_concat_column, **kw)} "
+        f"separator {compiler.process(separator, **kw)})"
     )
 
 
-def _parse_metadata_fallback(metadata):
+def _parse_metadata_fallback(metadata: Any) -> Iterator[tuple[str, str]]:
     yield ("unnamed", str(metadata))
 
 
-def _parse_metadata(metadata):
+def _parse_metadata(metadata: str) -> Iterator[tuple[str, str]]:
     try:
         parsed = json.loads(metadata)
     except json.decoder.JSONDecodeError:
@@ -153,24 +159,13 @@ def _parse_metadata(metadata):
         yield (key, str(value))
 
 
-def is_head(db_url, upgrade=False):
-    """Check whether or not db_url is head.
-
-    Args:
-        db_url (str): database url
-        upgrade (Bool): if True, upgrade db to head
-    """
-    engine = create_engine(db_url)
+def _is_head(db_url: str, upgrade=False) -> bool:
+    """Check whether the database at db_url is at the head revision."""
+    engine = create_engine(db_url, future=True)
     return is_head_engine(engine, upgrade=upgrade)
 
 
-def is_head_engine(engine, upgrade=False):
-    """Check whether or not engine is head.
-
-    Args:
-        engine (Engine): database engine
-        upgrade (Bool): if True, upgrade db to head
-    """
+def is_head_engine(engine: Engine, upgrade: bool = False) -> bool:
     config = Config()
     config.set_main_option("script_location", "spinedb_api:alembic")
     script = ScriptDirectory.from_config(config)
@@ -182,6 +177,7 @@ def is_head_engine(engine, upgrade=False):
             return True
         if not upgrade:
             return False
+
         # Upgrade function
         def fn(rev, context):
             return script._upgrade_revs("head", rev)
@@ -195,25 +191,48 @@ def is_head_engine(engine, upgrade=False):
     return True
 
 
-def copy_database(dest_url, source_url, overwrite=True, upgrade=False, only_tables=(), skip_tables=()):
-    """Copy the database from source_url into dest_url."""
-    if not is_head(source_url, upgrade=upgrade):
+def copy_database(
+    dest_url: str,
+    source_url: str,
+    overwrite: bool = True,
+    upgrade: bool = False,
+    only_tables: Sequence[str] = (),
+    skip_tables: Sequence[str] = (),
+) -> None:
+    """Copy the database from one url to another.
+
+    Args:
+        dest_url: The destination url.
+        source_url: The source url.
+        overwrite: whether to overwrite the destination.
+        upgrade: whether to upgrade the source to the latest Spine schema revision.
+        only_tables: If given, only these tables are copied.
+        skip_tables: If given, these tables are skipped.
+    """
+    if not _is_head(source_url, upgrade=upgrade):
         raise SpineDBVersionError(url=source_url)
-    source_engine = create_engine(source_url)
-    dest_engine = create_engine(dest_url)
+    source_engine = create_engine(source_url, future=True)
+    dest_engine = create_engine(dest_url, future=True)
     copy_database_bind(
         dest_engine,
         source_engine,
         overwrite=overwrite,
-        upgrade=upgrade,
         only_tables=only_tables,
         skip_tables=skip_tables,
     )
+    source_engine.dispose()
+    dest_engine.dispose()
 
 
-def copy_database_bind(dest_bind, source_bind, overwrite=True, upgrade=False, only_tables=(), skip_tables=()):
-    source_meta = MetaData(bind=source_bind)
-    source_meta.reflect()
+def copy_database_bind(
+    dest_bind: Engine,
+    source_bind: Engine,
+    overwrite: bool = True,
+    only_tables: Sequence[str] = (),
+    skip_tables: Sequence[str] = (),
+) -> None:
+    source_meta = MetaData()
+    source_meta.reflect(bind=source_bind)
     if inspect(dest_bind).get_table_names():
         if not overwrite:
             raise SpineDBAPIError(
@@ -222,54 +241,24 @@ def copy_database_bind(dest_bind, source_bind, overwrite=True, upgrade=False, on
                 "to the function call."
             )
         source_meta.drop_all(dest_bind)
-    dest_meta = MetaData(bind=dest_bind)
-    for source_table in source_meta.sorted_tables:
-        # Create table in dest
-        source_table.create(dest_bind)
-        if source_table.name not in ("alembic_version", "next_id"):
-            # Skip tables according to `only_tables` and `skip_tables`
-            if only_tables and source_table.name not in only_tables:
+    for table in source_meta.sorted_tables:
+        table.create(dest_bind)
+        if table.name not in ("alembic_version", "next_id"):
+            if (only_tables and table.name not in only_tables) or table.name in skip_tables:
                 continue
-            if source_table.name in skip_tables:
-                continue
-        dest_table = Table(source_table, dest_meta, autoload=True)
-        sel = select([source_table])
-        result = source_bind.execute(sel)
-        # Insert data from source into destination
-        data = result.fetchall()
+        with source_bind.begin() as connection:
+            result = connection.execute(select(table))
+            data = list(row._asdict() for row in result.fetchall())
         if not data:
             continue
-        ins = dest_table.insert()
-        try:
-            dest_bind.execute(ins, data)
-        except IntegrityError as e:
-            warnings.warn("Skipping table {0}: {1}".format(source_table.name, e.orig.args))
+        with dest_bind.begin() as connection:
+            try:
+                connection.execute(table.insert(), data)
+            except IntegrityError as e:
+                warnings.warn(f"Skipping table {table.name}: {e.orig.args}")
 
 
-def custom_generate_relationship(base, direction, return_fn, attrname, local_cls, referred_cls, **kw):
-    """Make all relationships view only to avoid warnings."""
-    kw["viewonly"] = True
-    kw["cascade"] = ""
-    kw["passive_deletes"] = False
-    kw["sync_backref"] = False
-    return generate_relationship(base, direction, return_fn, attrname, local_cls, referred_cls, **kw)
-
-
-def is_unlocked(db_url, timeout=0):
-    """Return True if the SQLite db_url is unlocked, after waiting at most timeout seconds.
-    Otherwise return False."""
-    if not db_url.startswith("sqlite"):
-        return False
-    try:
-        engine = create_engine(db_url, connect_args={"timeout": timeout})
-        engine.execute("BEGIN IMMEDIATE")
-        return True
-    except OperationalError:
-        return False
-
-
-def compare_schemas(left_engine, right_engine):
-    """Whether or not the left and right engine have the same schema."""
+def compare_schemas(left_engine: Engine, right_engine: Engine) -> bool:
     left_insp = inspect(left_engine)
     right_insp = inspect(right_engine)
     left_dict = schema_dict(left_insp)
@@ -277,7 +266,7 @@ def compare_schemas(left_engine, right_engine):
     return str(left_dict) == str(right_dict)
 
 
-def schema_dict(insp):
+def schema_dict(insp) -> dict:
     return {
         table_name: {
             "columns": sorted(insp.get_columns(table_name), key=itemgetter("name")),
@@ -289,18 +278,25 @@ def schema_dict(insp):
     }
 
 
-def is_empty(db_url):
+def is_empty(db_url: str) -> bool:
     try:
-        engine = create_engine(db_url)
+        engine = create_engine(db_url, future=True)
     except DatabaseError as e:
-        raise SpineDBAPIError("Could not connect to '{}': {}".format(db_url, e.orig.args)) from None
+        raise SpineDBAPIError(f"Could not connect to '{db_url}': {e.orig.args}") from None
     insp = inspect(engine)
     if insp.get_table_names():
         return False
     return True
 
 
-def create_spine_metadata():
+def get_head_alembic_version() -> str:
+    config = Config()
+    config.set_main_option("script_location", "spinedb_api:alembic")
+    script = ScriptDirectory.from_config(config)
+    return script.get_current_head()
+
+
+def create_spine_metadata() -> MetaData:
     meta = MetaData(naming_convention=naming_convention)
     Table(
         "commit",
@@ -347,133 +343,87 @@ def create_spine_metadata():
         UniqueConstraint("scenario_id", "rank"),
         UniqueConstraint("scenario_id", "alternative_id"),
     )
-    Table(
-        "entity_class_type",
-        meta,
-        Column("id", Integer, primary_key=True),
-        Column("name", String(255), nullable=False),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-    )
-    Table(
-        "entity_type",
-        meta,
-        Column("id", Integer, primary_key=True),
-        Column("name", String(255), nullable=False),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-    )
     Table(
         "entity_class",
         meta,
         Column("id", Integer, primary_key=True),
-        Column(
-            "type_id",
-            Integer,
-            ForeignKey("entity_class_type.id", onupdate="CASCADE", ondelete="CASCADE"),
-            nullable=False,
-        ),
         Column("name", String(255), nullable=False),
         Column("description", Text(), server_default=null()),
         Column("display_order", Integer, server_default="99"),
         Column("display_icon", BigInteger, server_default=null()),
         Column("hidden", Integer, server_default="0"),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-        UniqueConstraint("id", "type_id"),
-        UniqueConstraint("type_id", "name"),
+        Column("active_by_default", Boolean(name="active_by_default"), server_default=true(), nullable=False),
     )
     Table(
-        "object_class",
+        "superclass_subclass",
         meta,
-        Column("entity_class_id", Integer, primary_key=True),
-        Column("type_id", Integer, nullable=False),
-        ForeignKeyConstraint(
-            ("entity_class_id", "type_id"), ("entity_class.id", "entity_class.type_id"), ondelete="CASCADE"
+        Column("id", Integer, primary_key=True),
+        Column(
+            "superclass_id",
+            Integer,
+            ForeignKey("entity_class.id", onupdate="CASCADE", ondelete="CASCADE"),
+            nullable=False,
+        ),
+        Column(
+            "subclass_id",
+            Integer,
+            ForeignKey("entity_class.id", onupdate="CASCADE", ondelete="CASCADE"),
+            nullable=False,
+            unique=True,
         ),
-        CheckConstraint("`type_id` = 1", name="type_id"),  # make sure object class can only have object type
     )
     Table(
-        "relationship_class",
-        meta,
-        Column("entity_class_id", Integer, primary_key=True),
-        Column("type_id", Integer, nullable=False),
-        ForeignKeyConstraint(
-            ("entity_class_id", "type_id"), ("entity_class.id", "entity_class.type_id"), ondelete="CASCADE"
-        ),
-        CheckConstraint("`type_id` = 2", name="type_id"),
-    )
-    Table(
-        "relationship_entity_class",
+        "entity_class_dimension",
         meta,
         Column(
             "entity_class_id",
             Integer,
-            ForeignKey("relationship_class.entity_class_id", onupdate="CASCADE", ondelete="CASCADE"),
+            ForeignKey("entity_class.id", onupdate="CASCADE", ondelete="CASCADE"),
             primary_key=True,
         ),
-        Column("dimension", Integer, primary_key=True),
-        Column("member_class_id", Integer, nullable=False),
-        Column("member_class_type_id", Integer, nullable=False),
-        UniqueConstraint("entity_class_id", "dimension", "member_class_id", name="uq_relationship_entity_class"),
-        ForeignKeyConstraint(("member_class_id", "member_class_type_id"), ("entity_class.id", "entity_class.type_id")),
-        CheckConstraint("`member_class_type_id` != 2", name="member_class_type_id"),
+        Column(
+            "dimension_id",
+            Integer,
+            ForeignKey("entity_class.id", onupdate="CASCADE", ondelete="CASCADE"),
+            primary_key=True,
+        ),
+        Column("position", Integer, primary_key=True),
+        UniqueConstraint("entity_class_id", "dimension_id", "position", name="uq_entity_class_dimension"),
     )
     Table(
         "entity",
         meta,
         Column("id", Integer, primary_key=True),
-        Column("type_id", Integer, ForeignKey("entity_type.id", onupdate="CASCADE", ondelete="CASCADE")),
         Column("class_id", Integer, ForeignKey("entity_class.id", onupdate="CASCADE", ondelete="CASCADE")),
         Column("name", String(255), nullable=False),
         Column("description", Text(), server_default=null()),
         Column("commit_id", Integer, ForeignKey("commit.id")),
-        UniqueConstraint("id", "class_id"),
-        UniqueConstraint("id", "type_id", "class_id"),
         UniqueConstraint("class_id", "name"),
     )
     Table(
-        "object",
-        meta,
-        Column("entity_id", Integer, primary_key=True),
-        Column("type_id", Integer, nullable=False),
-        ForeignKeyConstraint(("entity_id", "type_id"), ("entity.id", "entity.type_id"), ondelete="CASCADE"),
-        CheckConstraint("`type_id` = 1", name="type_id"),  # make sure object can only have object type
-    )
-    Table(
-        "relationship",
+        "entity_element",
         meta,
         Column("entity_id", Integer, primary_key=True),
         Column("entity_class_id", Integer, nullable=False),
-        Column("type_id", Integer, nullable=False),
-        UniqueConstraint("entity_id", "entity_class_id"),
-        ForeignKeyConstraint(("entity_id", "type_id"), ("entity.id", "entity.type_id"), ondelete="CASCADE"),
-        CheckConstraint("`type_id` = 2", name="type_id"),
-    )
-    Table(
-        "relationship_entity",
-        meta,
-        Column("entity_id", Integer, primary_key=True),
-        Column("entity_class_id", Integer, nullable=False),
-        Column("dimension", Integer, primary_key=True),
-        Column("member_id", Integer, nullable=False),
-        Column("member_class_id", Integer, nullable=False),
+        Column("element_id", Integer, nullable=False),
+        Column("dimension_id", Integer, nullable=False),
+        Column("position", Integer, primary_key=True),
         ForeignKeyConstraint(
-            ("member_id", "member_class_id"), ("entity.id", "entity.class_id"), onupdate="CASCADE", ondelete="CASCADE"
+            ("entity_id", "entity_class_id"), ("entity.id", "entity.class_id"), onupdate="CASCADE", ondelete="CASCADE"
         ),
         ForeignKeyConstraint(
-            ("entity_class_id", "dimension", "member_class_id"),
+            ("element_id", "dimension_id"), ("entity.id", "entity.class_id"), onupdate="CASCADE", ondelete="CASCADE"
+        ),
+        ForeignKeyConstraint(
+            ("entity_class_id", "dimension_id", "position"),
             (
-                "relationship_entity_class.entity_class_id",
-                "relationship_entity_class.dimension",
-                "relationship_entity_class.member_class_id",
+                "entity_class_dimension.entity_class_id",
+                "entity_class_dimension.dimension_id",
+                "entity_class_dimension.position",
             ),
             onupdate="CASCADE",
             ondelete="CASCADE",
         ),
-        ForeignKeyConstraint(
-            ("entity_id", "entity_class_id"),
-            ("relationship.entity_id", "relationship.entity_class_id"),
-            onupdate="CASCADE",
-            ondelete="CASCADE",
-        ),
     )
     Table(
         "entity_group",
@@ -490,6 +440,59 @@ def create_spine_metadata():
             ("member_id", "entity_class_id"), ("entity.id", "entity.class_id"), onupdate="CASCADE", ondelete="CASCADE"
         ),
     )
+    Table(
+        "entity_alternative",
+        meta,
+        Column("id", Integer, primary_key=True),
+        Column("entity_id", Integer, ForeignKey("entity.id", onupdate="CASCADE", ondelete="CASCADE"), nullable=False),
+        Column(
+            "alternative_id",
+            Integer,
+            ForeignKey("alternative.id", onupdate="CASCADE", ondelete="CASCADE"),
+            nullable=False,
+        ),
+        Column("active", Boolean(name="active"), server_default=true(), nullable=False),
+        Column("commit_id", Integer, ForeignKey("commit.id")),
+        UniqueConstraint("entity_id", "alternative_id"),
+    )
+    Table(
+        "display_mode",
+        meta,
+        Column("id", Integer, primary_key=True),
+        Column("name", String(255), nullable=False, unique=True),
+        Column("description", Text(), server_default=null()),
+    )
+    Table(
+        "entity_class_display_mode",
+        meta,
+        Column("id", Integer, primary_key=True),
+        Column(
+            "display_mode_id",
+            Integer,
+            ForeignKey(
+                "display_mode.id",
+                onupdate="CASCADE",
+                ondelete="CASCADE",
+            ),
+            nullable=False,
+        ),
+        Column(
+            "entity_class_id",
+            Integer,
+            ForeignKey("entity_class.id", onupdate="CASCADE", ondelete="CASCADE"),
+            nullable=False,
+        ),
+        Column("display_order", Integer, nullable=False),
+        Column(
+            "display_status",
+            Enum(DisplayStatus, name="display_status_enum", create_constraint=True),
+            server_default=DisplayStatus.visible.name,
+            nullable=False,
+        ),
+        Column("display_font_color", String(6), server_default=null()),
+        Column("display_background_color", String(6), server_default=null()),
+        UniqueConstraint("display_mode_id", "entity_class_id"),
+    )
     Table(
         "parameter_definition",
         meta,
@@ -510,6 +513,20 @@ def create_spine_metadata():
         UniqueConstraint("entity_class_id", "name"),
         UniqueConstraint("id", "parameter_value_list_id"),
     )
+    Table(
+        "parameter_type",
+        meta,
+        Column("id", Integer, primary_key=True),
+        Column(
+            "parameter_definition_id",
+            Integer,
+            ForeignKey("parameter_definition.id", onupdate="CASCADE", ondelete="CASCADE"),
+            nullable=False,
+        ),
+        Column("rank", Integer, nullable=False),
+        Column("type", String(255), nullable=False),
+        UniqueConstraint("parameter_definition_id", "type", "rank"),
+    )
     Table(
         "parameter_tag",
         meta,
@@ -572,71 +589,6 @@ def create_spine_metadata():
         Column("commit_id", Integer, ForeignKey("commit.id")),
         UniqueConstraint("parameter_value_list_id", "index"),
     )
-    Table(
-        "tool",
-        meta,
-        Column("id", Integer, primary_key=True),
-        Column("name", String(155), nullable=False),
-        Column("description", Text(), server_default=null()),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-    )
-    Table(
-        "feature",
-        meta,
-        Column("id", Integer, primary_key=True),
-        Column("parameter_definition_id", Integer, nullable=False),
-        Column("parameter_value_list_id", Integer, nullable=False),
-        Column("description", Text(), server_default=null()),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-        UniqueConstraint("parameter_definition_id", "parameter_value_list_id"),
-        UniqueConstraint("id", "parameter_value_list_id"),
-        ForeignKeyConstraint(
-            ("parameter_definition_id", "parameter_value_list_id"),
-            ("parameter_definition.id", "parameter_definition.parameter_value_list_id"),
-            onupdate="CASCADE",
-            ondelete="CASCADE",
-        ),
-    )
-    Table(
-        "tool_feature",
-        meta,
-        Column("id", Integer, primary_key=True),
-        Column("tool_id", Integer, ForeignKey("tool.id")),
-        Column("feature_id", Integer, nullable=False),
-        Column("parameter_value_list_id", Integer, nullable=False),
-        Column("required", Boolean(name="required"), server_default=false(), nullable=False),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-        UniqueConstraint("tool_id", "feature_id"),
-        UniqueConstraint("id", "parameter_value_list_id"),
-        ForeignKeyConstraint(
-            ("feature_id", "parameter_value_list_id"),
-            ("feature.id", "feature.parameter_value_list_id"),
-            onupdate="CASCADE",
-            ondelete="CASCADE",
-        ),
-    )
-    Table(
-        "tool_feature_method",
-        meta,
-        Column("id", Integer, primary_key=True),
-        Column("tool_feature_id", Integer, nullable=False),
-        Column("parameter_value_list_id", Integer, nullable=False),
-        Column("method_index", Integer),
-        Column("commit_id", Integer, ForeignKey("commit.id")),
-        UniqueConstraint("tool_feature_id", "method_index"),
-        ForeignKeyConstraint(
-            ("tool_feature_id", "parameter_value_list_id"),
-            ("tool_feature.id", "tool_feature.parameter_value_list_id"),
-            onupdate="CASCADE",
-            ondelete="CASCADE",
-        ),
-        ForeignKeyConstraint(
-            ("parameter_value_list_id", "method_index"),
-            ("list_value.parameter_value_list_id", "list_value.index"),
-            onupdate="CASCADE",
-            ondelete="CASCADE",
-        ),
-    )
     Table(
         "metadata",
         meta,
@@ -673,6 +625,18 @@ def create_spine_metadata():
         Column("commit_id", Integer, ForeignKey("commit.id")),
         UniqueConstraint("entity_id", "metadata_id"),
     )
+    Table(
+        "entity_location",
+        meta,
+        Column("id", Integer, primary_key=True),
+        Column("entity_id", Integer, ForeignKey("entity.id", onupdate="CASCADE", ondelete="CASCADE"), nullable=False),
+        Column("lat", Float),
+        Column("lon", Float),
+        Column("alt", Float),
+        Column("shape_name", String(155)),
+        Column("shape_blob", Text, server_default=null()),
+        UniqueConstraint("entity_id"),
+    )
     Table(
         "alembic_version",
         meta,
@@ -682,41 +646,45 @@ def create_spine_metadata():
     return meta
 
 
-def create_new_spine_database(db_url):
+def create_new_spine_database(db_url: str) -> Engine:
     """Create a new Spine database at the given url."""
     try:
-        engine = create_engine(db_url)
+        engine = create_engine(db_url, future=True)
     except DatabaseError as e:
-        raise SpineDBAPIError("Could not connect to '{}': {}".format(db_url, e.orig.args)) from None
-    # Drop existing tables. This is a Spine db now...
-    meta = MetaData(engine)
-    meta.reflect()
-    meta.drop_all()
-    # Create new tables
-    meta = create_spine_metadata()
-    try:
-        meta.create_all(engine)
-        engine.execute("INSERT INTO `commit` VALUES (1, 'Create the database', CURRENT_TIMESTAMP, 'spinedb_api')")
-        engine.execute("INSERT INTO alternative VALUES (1, 'Base', 'Base alternative', 1)")
-        engine.execute("INSERT INTO entity_class_type VALUES (1, 'object', 1), (2, 'relationship', 1)")
-        engine.execute("INSERT INTO entity_type VALUES (1, 'object', 1), (2, 'relationship', 1)")
-        engine.execute("INSERT INTO alembic_version VALUES ('989fccf80441')")
-    except DatabaseError as e:
-        raise SpineDBAPIError("Unable to create Spine database: {}".format(e)) from None
+        raise SpineDBAPIError(f"Could not connect to '{db_url}': {e.orig.args}") from None
+    create_new_spine_database_from_engine(engine)
     return engine
 
 
-def _create_first_spine_database(db_url):
-    """Creates a Spine database with the very first version at the given url.
-    Used internally.
-    """
-    try:
-        engine = create_engine(db_url)
-    except DatabaseError as e:
-        raise SpineDBAPIError("Could not connect to '{}': {}".format(db_url, e.orig.args)) from None
+def create_new_spine_database_from_engine(engine: Engine) -> None:
     # Drop existing tables. This is a Spine db now...
-    meta = MetaData(engine)
-    meta.reflect()
+    meta = MetaData()
+    meta.reflect(engine)
+    meta.drop_all(engine)
+    # Create new tables
+    meta = create_spine_metadata()
+    version = get_head_alembic_version()
+    try:
+        meta.create_all(engine)
+        with engine.begin() as connection:
+            connection.execute(
+                text("INSERT INTO `commit` VALUES (1, 'Create the database', CURRENT_TIMESTAMP, 'spinedb_api')")
+            )
+            connection.execute(text("INSERT INTO alternative VALUES (1, 'Base', 'Base alternative', 1)"))
+            connection.execute(text(f"INSERT INTO alembic_version VALUES ('{version}')"))
+    except DatabaseError as e:
+        raise SpineDBAPIError(f"Unable to create Spine database: {e}") from None
+
+
+def _create_first_spine_database(db_url: str) -> Engine:
+    """Creates a Spine database with the very first version at the given url."""
+    try:
+        engine = create_engine(db_url, future=True)
+    except DatabaseError as e:
+        raise SpineDBAPIError(f"Could not connect to '{db_url}': {e.orig.args}") from None
+    # Drop existing tables. This is a Spine db now...
+    meta = MetaData()
+    meta.reflect(engine)
     meta.drop_all(engine)
     # Create new tables
     meta = MetaData(naming_convention=naming_convention)
@@ -855,18 +823,18 @@ def _create_first_spine_database(db_url):
     try:
         meta.create_all(engine)
     except DatabaseError as e:
-        raise SpineDBAPIError("Unable to create Spine database: {}".format(e.orig.args))
+        raise SpineDBAPIError(f"Unable to create Spine database: {e.orig.args}") from e
     return engine
 
 
-def forward_sweep(root, fn):
+def forward_sweep(root: SelectBase, fn: Callable[[SelectBase, ...], None], *args) -> None:
     """Recursively visit, using `get_children()`, the given sqlalchemy object.
     Apply `fn` on every visited node."""
     current = root
     parent = {}
     children = {current: iter(current.get_children(column_collections=False))}
     while True:
-        fn(current)
+        fn(current, *args)
         # Try and visit next children
         next_ = next(children[current], None)
         if next_ is not None:
@@ -884,40 +852,11 @@ def forward_sweep(root, fn):
             continue
         # No (more) siblings, go back to parent
         current = current_parent
-        if current == root:
+        if current is root:
             break
 
 
-def get_relationship_entity_class_items(item, object_class_type):
-    return [
-        {
-            "entity_class_id": item["id"],
-            "dimension": dimension,
-            "member_class_id": object_class_id,
-            "member_class_type_id": object_class_type,
-        }
-        for dimension, object_class_id in enumerate(item["object_class_id_list"])
-    ]
-
-
-def get_relationship_entity_items(item, relationship_entity_type, object_entity_type):
-    return [
-        {
-            "entity_id": item["id"],
-            "type_id": relationship_entity_type,
-            "entity_class_id": item["class_id"],
-            "dimension": dimension,
-            "member_id": object_id,
-            "member_class_type_id": object_entity_type,
-            "member_class_id": object_class_id,
-        }
-        for dimension, (object_id, object_class_id) in enumerate(
-            zip(item["object_id_list"], item["object_class_id_list"])
-        )
-    ]
-
-
-def labelled_columns(table):
+def labelled_columns(table: Table) -> list:
     return [c.label(c.name) for c in table.columns]
 
 
@@ -929,27 +868,29 @@ class AsteriskType:
 Asterisk = AsteriskType()
 
 
-def fix_name_ambiguity(input_list, offset=0, prefix=""):
+def fix_name_ambiguity(input_list: Sequence[str], offset: int = 0, prefix: str = "") -> list[str]:
     """Modify repeated entries in name list by appending an increasing integer."""
     result = []
-    ocurrences = {}
+    occurrences = {}
     for item in input_list:
-        n_ocurrences = input_list.count(item)
-        if n_ocurrences > 1:
-            ocurrence = ocurrences.get(item, 1)
-            ocurrences[item] = ocurrence + 1
-            item += prefix + str(offset + ocurrence)
+        n_occurrences = input_list.count(item)
+        if n_occurrences > 1:
+            occurrence = occurrences.get(item, 1)
+            occurrences[item] = occurrence + 1
+            item += prefix + str(offset + occurrence)
         result.append(item)
     return result
 
 
-def vacuum(url):
-    engine = create_engine(url)
+def vacuum(url: str) -> tuple[int, str]:
+    engine = create_engine(url, future=True)
     if not engine.url.drivername.startswith("sqlite"):
         return 0, "bytes"
     size_before = os.path.getsize(engine.url.database)
-    engine.execute("vacuum")
+    with engine.begin() as connection:
+        connection.execute(text("vacuum"))
     freed = size_before - os.path.getsize(engine.url.database)
+    engine.dispose()
     k = 0
     units = ("bytes", "KB", "MB", "GB", "TB")
     while freed > 1e3 and k < len(units):
@@ -958,16 +899,44 @@ def vacuum(url):
     return freed, units[k]
 
 
-def remove_credentials_from_url(url):
-    """Removes username and password information from URLs.
-
-    Args:
-        url (str): URL
-
-    Returns:
-        str: sanitized URL
-    """
-    parsed = urlparse(url)
-    if parsed.username is None:
+def remove_credentials_from_url(url: str) -> str:
+    """Removes username and password information from URLs."""
+    if "@" not in url:
         return url
-    return urlunparse(parsed._replace(netloc=parsed.netloc.partition("@")[-1]))
+    head, tail = url.rsplit("@", maxsplit=1)
+    scheme, _ = head.split("://", maxsplit=1)
+    return scheme + "://" + tail
+
+
+def group_consecutive(list_of_numbers: Iterable[int]) -> Iterator[tuple[int, int]]:
+    for _k, g in groupby(enumerate(sorted(list_of_numbers)), lambda x: x[0] - x[1]):
+        group = list(map(itemgetter(1), g))
+        yield group[0], group[-1]
+
+
+_TRUTHS = {s.casefold() for s in ("yes", "true", "y", "t", "1")}
+_FALSES = {s.casefold() for s in ("no", "false", "n", "f", "0")}
+
+
+def string_to_bool(string: str) -> bool:
+    """Converts string to boolean.
+
+    Recognizes "yes", "true", "y", "t" and "1" as True, "no", "false", "n", "f" and "0" as False.
+    Case-insensitive.
+    Raises Value error if value is not recognized.
+    """
+    string = string.casefold()
+    if string in _TRUTHS:
+        return True
+    if string in _FALSES:
+        return False
+    raise ValueError(string)
+
+
+@enum.unique
+class DisplayStatus(enum.Enum):
+    """Custom enum for entity class display status."""
+
+    visible = enum.auto()
+    hidden = enum.auto()
+    greyed_out = enum.auto()
