@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -12,26 +13,25 @@
 """
 Contains `get_mapped_data()` that converts rows of tabular data into a dictionary for import to a Spine DB,
 using ``import_functions.import_data()``
-
-:author: P. Vennström (VTT)
-:date:   22.02.2018
 """
-
+from collections.abc import Callable
 from copy import deepcopy
-from .import_mapping_compat import import_mapping_from_dict
-from .import_mapping import ImportMapping, check_validity
-from ..mapping import Position
+from operator import itemgetter
+from typing import Any, Optional
+from ..exception import ParameterValueFormatError
+from ..helpers import string_to_bool
+from ..mapping import Position, is_pivoted
 from ..parameter_value import (
-    convert_leaf_maps_to_specialized_containers,
-    Map,
-    TimeSeriesVariableResolution,
-    TimePattern,
     Array,
+    Map,
+    TimePattern,
+    TimeSeriesVariableResolution,
+    convert_leaf_maps_to_specialized_containers,
     from_database,
     split_value_and_type,
 )
-from ..exception import ParameterValueFormatError
-
+from .import_mapping import ImportMapping, check_validity
+from .import_mapping_compat import import_mapping_from_dict
 
 _NO_VALUE = object()
 
@@ -57,6 +57,7 @@ def get_mapped_data(
     default_column_convert_fn=None,
     row_convert_fns=None,
     unparse_value=identity,
+    mapping_names=None,
 ):
     """
     Args:
@@ -68,6 +69,7 @@ def get_mapped_data(
         default_column_convert_fn (Callable, optional): default convert function for surplus columns
         row_convert_fns (dict(int,function), optional): mapping from row number to convert function
         unparse_value (Callable): a callable that converts values to database format
+        mapping_names (list, optional): list of mapping names (order corresponds to order of mappings).
 
     Returns:
         dict: Mapped data, ready for ``import_data()``
@@ -84,43 +86,57 @@ def get_mapped_data(
     rows = list(data_source)
     if not rows:
         return mapped_data, errors
+    column_count = len(max(rows, key=lambda x: len(x) if x else 0))
     if column_convert_fns is None:
         column_convert_fns = {}
     if row_convert_fns is None:
         row_convert_fns = {}
     if default_column_convert_fn is None:
-        default_column_convert_fn = column_convert_fns[max(column_convert_fns)] if column_convert_fns else identity
-    for mapping in mappings:
-        read_state = {}
+        default_column_convert_fn = _last_column_convert_function(column_convert_fns)
+    if mapping_names is None:
+        mapping_names = []
+    _ensure_mapping_name_consistency(mappings, mapping_names)
+    for mapping, mapping_name in zip(mappings, mapping_names):
         mapping = deepcopy(mapping)
-        mapping.polish(table_name, data_header)
+        mapping.polish(table_name, data_header, mapping_name, column_count)
         mapping_errors = check_validity(mapping)
         if mapping_errors:
             errors += mapping_errors
             continue
+        read_state = {}
         # Find pivoted and unpivoted mappings
         pivoted, non_pivoted, pivoted_from_header, last = _split_mapping(mapping)
         # If there are no pivoted mappings, we can just feed the rows to our mapping directly
         if not (pivoted or pivoted_from_header):
             start_pos = mapping.read_start_row
-            for k, row in enumerate(rows[mapping.read_start_row :]):
+            for k, row in enumerate(rows[start_pos:]):
                 if not _is_valid_row(row):
                     continue
                 row = _convert_row(row, column_convert_fns, start_pos + k, errors)
                 mapping.import_row(row, read_state, mapped_data)
             continue
         # There are pivoted mappings. We unpivot the table
+        pivoted_by_leaf = all(
+            not is_pivoted(m.position) and m.position != Position.header for m in mapping.flatten()[:-1]
+        )
         unpivoted_rows, pivoted_pos, non_pivoted_pos, unpivoted_column_pos = _unpivot_rows(
-            rows, data_header, pivoted, non_pivoted, pivoted_from_header, mapping.skip_columns
+            rows,
+            data_header,
+            pivoted,
+            non_pivoted,
+            pivoted_from_header,
+            mapping.skip_columns,
+            mapping.read_start_row,
+            pivoted_by_leaf,
         )
         if not unpivoted_column_pos:
             continue
+        last.position = -1
         # Reposition row convert functions
         row_convert_fns = {k: row_convert_fns[pos] for k, pos in enumerate(pivoted_pos) if pos in row_convert_fns}
         # If there are only pivoted mappings, we can just feed the unpivoted rows
         if not non_pivoted:
             # Reposition pivoted mappings:
-            last.position = -1
             for k, m in enumerate(pivoted):
                 m.position = k
             for k, row in enumerate(unpivoted_rows):
@@ -134,7 +150,6 @@ def get_mapped_data(
         # - The last mapping (typically, parameter value) will read from the last position in the row
         # - The pivoted mappings will read from positions to the left of that
         k = None
-        last.position = -1
         for k, m in enumerate(reversed(pivoted)):
             m.position = -(k + 2)
         # Feed rows: To each regular row, we append each unpivoted row, plus the item at the intersection,
@@ -159,11 +174,19 @@ def get_mapped_data(
                 full_row = non_pivoted_row + unpivoted_row
                 full_row.append(row[column_pos])
                 mapping.import_row(full_row, read_state, mapped_data)
-    _make_relationship_classes(mapped_data)
+    _make_entity_classes(mapped_data)
+    _make_entities(mapped_data)
+    _make_entity_metadata(mapped_data)
+    _make_entity_alternatives(mapped_data, errors)
     _make_parameter_values(mapped_data, unparse_value)
+    _make_parameter_value_metadata(mapped_data)
     return mapped_data, errors
 
 
+def _last_column_convert_function(functions: Optional[dict]) -> Callable[[Any], Any]:
+    return functions[max(functions)] if functions else identity
+
+
 def _is_valid_row(row):
     return row is not None and not all(i is None for i in row)
 
@@ -201,7 +224,7 @@ def _split_mapping(mapping):
     non_pivoted = []
     pivoted_from_header = []
     for m in flattened:
-        if pivoted and m is flattened[-1]:
+        if (pivoted or pivoted_from_header) and m is flattened[-1]:
             # If any other mapping is pivoted, ignore last mapping's position
             break
         if m.position == Position.header and m.value is None:
@@ -216,7 +239,9 @@ def _split_mapping(mapping):
     return pivoted, non_pivoted, pivoted_from_header, flattened[-1]
 
 
-def _unpivot_rows(rows, data_header, pivoted, non_pivoted, pivoted_from_header, skip_columns):
+def _unpivot_rows(
+    rows, data_header, pivoted, non_pivoted, pivoted_from_header, skip_columns, read_start_row, pivoted_by_leaf
+):
     """Unpivots rows.
 
     Args:
@@ -225,6 +250,9 @@ def _unpivot_rows(rows, data_header, pivoted, non_pivoted, pivoted_from_header,
         pivoted (list of ImportMapping): Pivoted mappings (reading from rows)
         non_pivoted (list of ImportMapping): Non-pivoted mappings ('regular', reading from columns)
         pivoted_from_header (list of ImportMapping): Mappings pivoted from header
+        skip_columns (list of int): columns that should be skipped
+        read_start_row (int): first row to include
+        pivoted_by_leaf (bool): whether only the leaf mapping is pivoted
 
     Returns:
         list of list: Unpivoted rows
@@ -236,7 +264,10 @@ def _unpivot_rows(rows, data_header, pivoted, non_pivoted, pivoted_from_header,
     pivoted_pos = [-(m.position + 1) for m in pivoted]  # (-1) -> (0), (-2) -> (1), (-3) -> (2), etc.
     non_pivoted_pos = [m.position for m in non_pivoted]
     # Collect pivoted rows
-    pivoted_rows = [rows[pos] for pos in pivoted_pos] if non_pivoted_pos else rows
+    if not pivoted_by_leaf:
+        pivoted_rows = [rows[pos] for pos in pivoted_pos] if non_pivoted_pos else rows
+    else:
+        pivoted_rows = [rows[pos + read_start_row] for pos in pivoted_pos] if non_pivoted_pos else rows[read_start_row:]
     # Prepend as many headers as needed
     for m in pivoted_from_header:
         pivoted.insert(0, m)
@@ -264,22 +295,48 @@ def _unpivot_rows(rows, data_header, pivoted, non_pivoted, pivoted_from_header,
     return unpivoted_rows, pivoted_pos, non_pivoted_pos, unpivoted_column_pos
 
 
-def _make_relationship_classes(mapped_data):
-    rows = mapped_data.get("relationship_classes")
+def _make_entity_classes(mapped_data):
+    rows = mapped_data.get("entity_classes")
     if rows is None:
         return
-    full_rows = []
-    for class_name, object_classes in rows.items():
-        full_rows.append((class_name, object_classes))
-    mapped_data["relationship_classes"] = full_rows
+    rows = [(class_name, tuple(dimension_names)) for class_name, dimension_names in rows.items()]
+    rows.sort(key=itemgetter(1))
+    mapped_data["entity_classes"] = final_rows = []
+    for class_name, dimension_names in rows:
+        row = (class_name, tuple(dimension_names)) if dimension_names else (class_name,)
+        final_rows.append(row)
+
+
+def _make_entities(mapped_data):
+    rows = mapped_data.get("entities")
+    if rows is None:
+        return
+    mapped_data["entities"] = list(rows)
+
+
+def _make_entity_alternatives(mapped_data, errors):
+    if "entity_alternatives" not in mapped_data:
+        return
+    rows = []
+    for item in mapped_data["entity_alternatives"]:
+        entity_class, byname, alternative, activity = item
+        try:
+            activity = string_to_bool(activity) if isinstance(activity, str) else bool(activity)
+        except ValueError:
+            errors.append(
+                f"Can't convert {activity} to entity alternative activity boolean "
+                f"for '{byname}' in '{entity_class}' with alternative '{alternative}'"
+            )
+        else:
+            rows.append((entity_class, byname, alternative, activity))
+    mapped_data["entity_alternatives"] = rows
 
 
 def _make_parameter_values(mapped_data, unparse_value):
     value_pos = 3
-    for key in ("object_parameter_values", "relationship_parameter_values"):
-        rows = mapped_data.get(key)
-        if rows is None:
-            continue
+    key = "parameter_values"
+    rows = mapped_data.get(key)
+    if rows is not None:
         valued_rows = []
         for row in rows:
             raw_value = _make_value(row, value_pos)
@@ -291,10 +348,9 @@ def _make_parameter_values(mapped_data, unparse_value):
                 valued_rows.append(row)
         mapped_data[key] = valued_rows
     value_pos = 0
-    for key in ("object_parameters", "relationship_parameters"):
-        rows = mapped_data.get(key)
-        if rows is None:
-            continue
+    key = "parameter_definitions"
+    rows = mapped_data.get(key)
+    if rows is not None:
         full_rows = []
         for entity_definition, extras in rows.items():
             if extras:
@@ -307,6 +363,20 @@ def _make_parameter_values(mapped_data, unparse_value):
         mapped_data[key] = full_rows
 
 
+def _make_parameter_value_metadata(mapped_data):
+    rows = mapped_data.get("parameter_value_metadata")
+    if rows is None:
+        return
+    mapped_data["parameter_value_metadata"] = list(rows)
+
+
+def _make_entity_metadata(mapped_data):
+    rows = mapped_data.get("entity_metadata")
+    if rows is None:
+        return
+    mapped_data["entity_metadata"] = list(rows)
+
+
 def _make_value(row, value_pos):
     try:
         value = row[value_pos]
@@ -354,21 +424,21 @@ def _table_to_map(table, compress=False):
 
 
 def _table_to_dict(table):
-    map_dict = dict()
+    map_dict = {}
     for row in table:
         row = [item for item in row if item not in (None, "")]
         if len(row) < 2:
             continue
         d = map_dict
         for item in row[:-2]:
-            d = d.setdefault(item, dict())
+            d = d.setdefault(item, {})
         d[row[-2]] = row[-1]
     return map_dict
 
 
 def _dict_to_map_recursive(d):
-    indexes = list()
-    values = list()
+    indexes = []
+    values = []
     for key, value in d.items():
         if isinstance(value, dict):
             value = _dict_to_map_recursive(value)
@@ -392,3 +462,18 @@ def _apply_index_names(map_, index_names):
     for v in map_.values:
         if isinstance(v, Map):
             _apply_index_names(v, index_names[1:])
+
+
+def _ensure_mapping_name_consistency(mappings, mapping_names):
+    """Makes sure that there are as many mapping names as actual mappings.
+
+    Args:
+        mappings (list(ImportMapping)): list of mappings
+        mapping_names (list(str)): list of mapping names
+    """
+    n_mappings = len(mappings)
+    n_mapping_names = len(mapping_names)
+    if n_mapping_names > n_mappings:
+        mapping_names = mapping_names[:n_mappings]
+    elif n_mapping_names < n_mappings:
+        mapping_names += [""] * (n_mappings - n_mapping_names)
