@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -9,15 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Type conversion functions.
-
-:author: P. Vennström (VTT)
-:date:   21.11.2019
-"""
+""" Type conversion functions. """
 
 import re
-from distutils.util import strtobool
+from spinedb_api.helpers import string_to_bool
 from spinedb_api.parameter_value import DateTime, Duration, ParameterValueFormatError
 
 
@@ -46,7 +42,12 @@ class ConvertSpec:
     RETURN_TYPE = str
 
     def __call__(self, value):
-        return self.RETURN_TYPE(value)
+        try:
+            return self.RETURN_TYPE(value)
+        except ValueError as error:
+            if not value:
+                return None
+            raise error
 
     def to_json_value(self):
         return self.DISPLAY_NAME
@@ -77,7 +78,7 @@ class BooleanConvertSpec(ConvertSpec):
     RETURN_TYPE = bool
 
     def __call__(self, value):
-        return self.RETURN_TYPE(strtobool(str(value)))
+        return self.RETURN_TYPE(string_to_bool(str(value)))
 
 
 class IntegerSequenceDateTimeConvertSpec(ConvertSpec):
@@ -103,8 +104,8 @@ class IntegerSequenceDateTimeConvertSpec(ConvertSpec):
             int_str = pattern.search(str(value)).group()
             int_value = int(int_str) - start_int
             return DateTime(start_datetime + int_value * duration)
-        except (ValueError, ParameterValueFormatError):
-            raise ValueError(f"Could not convert '{value}' to a DateTime")
+        except (ValueError, ParameterValueFormatError) as error:
+            raise ValueError(f"Could not convert '{value}' to a DateTime") from error
 
     def to_json_value(self):
         return {
