@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -8,25 +9,28 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""
-Contains export mappings for database items such as entities, entity classes and parameter values.
 
-:author: A. Soininen (VTT)
-:date:   10.12.2020
-"""
+"""Base class for import and export mappings."""
 
 from enum import Enum, unique
 from itertools import takewhile
 import re
+from typing import Optional
+from spinedb_api import InvalidMapping
+
+_TABLEFUL_FIXED_POSITION_RE = re.compile(r"^\s*(.+):\s*(\d+)\s*,\s*(\d+)\s*$")
+_TABLELESS_FIXED_POSITION_RE = re.compile(r"^\s*(\d+)\s*,\s*(\d+)\s*$")
 
 
 @unique
 class Position(Enum):
-    """Export item positions when they are not in columns."""
+    """Item positions when they are not in columns."""
 
     hidden = "hidden"
     table_name = "table_name"
     header = "header"
+    mapping_name = "mapping_name"
+    fixed = "fixed"
 
 
 def is_pivoted(position):
@@ -53,6 +57,21 @@ def is_regular(position):
     return isinstance(position, int) and position >= 0
 
 
+def parse_fixed_position_value(value: str) -> tuple[Optional[str], int, int]:
+    """Parses mapping's value for ``Position.fixed`` to table name, row and column."""
+    match = re.search(_TABLEFUL_FIXED_POSITION_RE, value)
+    if match:
+        return match.group(1), int(match.group(2)), int(match.group(3))
+    match = re.search(_TABLELESS_FIXED_POSITION_RE, value)
+    if match:
+        return None, int(match.group(1)), int(match.group(2))
+    raise InvalidMapping(f"failed to parse fixed position '{value}', expected 'table_name: row, column")
+
+
+def unparse_fixed_position_value(table_name: Optional[str], row: int, column: int) -> str:
+    return (f"{table_name}: " if table_name is not None else "") + f"{row}, {column}"
+
+
 class Mapping:
     """Base class for import/export item mappings.
 
@@ -161,6 +180,16 @@ class Mapping:
         """
         return [self] + (self.child.flatten() if self.child is not None else [])
 
+    def is_effective_leaf(self):
+        """Tests if mapping is effectively the leaf mapping.
+
+        Returns:
+            bool: True if mapping is effectively the last child, False otherwise
+        """
+        return self._child is None or all(
+            child.position in (Position.hidden, Position.table_name) for child in self._child.flatten()[:-1]
+        )
+
     def is_pivoted(self):
         """
         Queries recursively if export items are pivoted.
@@ -211,7 +240,7 @@ class Mapping:
 
     def last_pivot_row(self):
         return max(
-            [-(m.position + 1) for m in self.flatten() if isinstance(m.position, int) and m.position < 0], default=-1
+            (-(m.position + 1) for m in self.flatten() if isinstance(m.position, int) and m.position < 0), default=-1
         )
 
     def query_parents(self, what):
