@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -11,8 +12,6 @@
 """
 Module contains a .csv writer implementation.
 
-:author: A. Soininen (VTT)
-:date:   9.12.2020
 """
 import csv
 import os
@@ -60,7 +59,7 @@ class CsvWriter(Writer):
         self._file_name = os.path.join(self._path, table_name)
         if self._file_name not in self._finished_files and os.path.exists(self._file_name):
             os.remove(self._file_name)
-        self._file = open(self._file_name, "a", newline="")
+        self._file = open(self._file_name, "a", newline="", encoding="utf-8")
         self._out = csv.writer(self._file)
         return True
 
