@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -8,20 +9,20 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""
-Module contains a .gdx writer implementation.
-
-:author: A. Soininen (VTT)
-:date:   9.12.2020
-"""
+""" Module contains a .gdx writer implementation. """
 import math
-from gdx2py import GAMSSet, GAMSScalar, GAMSParameter, GdxFile
+from gams.core import gdx
+from gdx2py import GAMSParameter, GAMSScalar, GAMSSet, GdxFile
 from gdx2py.gdxfile import EPS_VALUE
-import gdxcc
+from ..gdx_utils import gams_supports_new_api
 from .writer import Writer, WriterException
 
-
-SPECIAL_CONVERSIONS = {EPS_VALUE: gdxcc.GMS_SV_EPS, math.inf: gdxcc.GMS_SV_PINF, -math.inf: gdxcc.GMS_SV_MINF}
+SPECIAL_CONVERSIONS = {
+    EPS_VALUE: gdx.GMS_SV_EPS,
+    1e-10: gdx.GMS_SV_EPS,
+    math.inf: gdx.GMS_SV_PINF,
+    -math.inf: gdx.GMS_SV_MINF,
+}
 
 
 class GdxWriter(Writer):
@@ -34,9 +35,11 @@ class GdxWriter(Writer):
         super().__init__()
         self._file_path = file_path
         self._gams_dir = gams_directory
+        if not gams_supports_new_api(self._gams_dir):
+            raise WriterException("GAMS version 42 or greater is required to write .gdx files")
         self._gdx_file = None
-        self._tables = dict()
-        self._table_dimensions = dict()
+        self._tables = {}
+        self._table_dimensions = {}
         self._current_table_name = None
         self._current_table = None
         self._dimensions_missing = True
@@ -52,14 +55,14 @@ class GdxWriter(Writer):
     def finish_table(self):
         if self._current_table_name is None:
             return
-        self._tables.setdefault(self._current_table_name, list()).extend(self._current_table)
+        self._tables.setdefault(self._current_table_name, []).extend(self._current_table)
         self._current_table_name = None
 
     def start(self):
         try:
             self._gdx_file = GdxFile(self._file_path, "w", self._gams_dir)
         except RuntimeError as e:
-            raise WriterException(f"Could not open .gdx file : {e}")
+            raise WriterException(f"Could not open .gdx file : {e}") from e
 
     def start_table(self, table_name, title_key):
         if not table_name:
@@ -67,7 +70,7 @@ class GdxWriter(Writer):
         if table_name in self._gdx_file:
             raise WriterException("Gdx does not support appending data to existing sets.")
         self._current_table_name = table_name
-        self._current_table = list()
+        self._current_table = []
         self._dimensions_missing = True
         return True
 
@@ -109,18 +112,18 @@ def _table_to_gdx(gdx_file, table, table_name, dimensions):
             try:
                 set_ = GAMSSet(table, dimensions)
             except ValueError as e:
-                raise WriterException(f"Error writing empty table '{table_name}': {e}")
+                raise WriterException(f"Error writing empty table '{table_name}': {e}") from e
     else:
         set_ = GAMSParameter({}, dimensions[:-1]) if is_parameter else GAMSSet(table, dimensions)
     try:
         gdx_file[table_name] = set_
     except TypeError as e:
         if isinstance(set_, GAMSSet):
-            raise WriterException(f"A column contains a mixture of numeric and non-numeric elements.")
+            raise WriterException("A column contains a mixture of numeric and non-numeric elements.") from e
         raise e
     except ValueError as e:
         if isinstance(set_, GAMSParameter):
-            raise WriterException(f"Failed to create GAMS parameter in table '{table_name}': {e}")
+            raise WriterException(f"Failed to create GAMS parameter in table '{table_name}': {e}") from e
         raise e
 
 
@@ -128,13 +131,15 @@ def _convert_to_gams(x):
     """Converts special float values to corresponding GAMS constants, otherwise returns x as is.
 
     Args:
-        x (float): value to convert
+        x (float or Any): value to convert
 
     Returns:
-        float: converted value
+        float or Any: converted value
     """
     if not isinstance(x, float):
+        if x == "EPS":
+            return gdx.GMS_SV_EPS
         return x
     if math.isnan(x):
-        return gdxcc.GMS_SV_UNDEF
+        return gdx.GMS_SV_UNDEF
     return SPECIAL_CONVERSIONS.get(x, x)
