@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -9,23 +10,19 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains ExcelConnector class and a help function.
+""" Contains ExcelReader class and helper functions. """
 
-:author: P. Vennström (VTT)
-:date:   1.6.2019
-"""
-
-from itertools import islice, takewhile, chain
 import io
+from itertools import chain, islice, takewhile
+from typing import Any
 from openpyxl import load_workbook
-from .reader import SourceConnection
+from ...exception import ReaderError
+from .reader import Reader, TableProperties
 
 
-class ExcelConnector(SourceConnection):
-    """Template class to read data from another QThread."""
+class ExcelReader(Reader):
+    """A reader for .xlsx files."""
 
-    # name of data source, ex: "Text/CSV"
     DISPLAY_NAME = "Excel"
 
     # dict with option specification for source.
@@ -44,11 +41,12 @@ class ExcelConnector(SourceConnection):
         self._filename = None
         self._wb = None
 
-    def connect_to_source(self, source):
-        """saves filepath
+    def connect_to_source(self, source, **extras):
+        """Connects to Excel file.
 
-        Arguments:
-            source {str} -- filepath
+        Args:
+            source (str): path to file
+            **extras: ignored
         """
         if source:
             self._filename = source
@@ -72,7 +70,7 @@ class ExcelConnector(SourceConnection):
         table_mappings = default_mapping["table_mappings"] = {}
         table_options = default_mapping["table_options"] = {}
         selected_tables = default_mapping["selected_tables"] = []
-        for sheet in self.get_tables():
+        for sheet in self.get_tables_and_properties():
             map_dict, option = create_mapping_from_sheet(self._wb[sheet])
             if map_dict:
                 table_mappings[sheet] = [map_dict]
@@ -81,21 +79,11 @@ class ExcelConnector(SourceConnection):
                 table_options[sheet] = option
         return default_mapping
 
-    def get_tables(self):
-        """Method that should return Excel sheets as mappings and their options.
-
-        Returns:
-            dict: Sheets as mappings and options for each sheet or an empty dictionary if no workbook.
-
-        Raises:
-            Exception: If something goes wrong.
-        """
+    def get_tables_and_properties(self):
+        """Returns sheet names and their options."""
         if not self._wb:
             return {}
-        try:
-            return self._wb.sheetnames
-        except Exception as error:
-            raise error
+        return {name: TableProperties() for name in self._wb.sheetnames}
 
     def get_data_iterator(self, table, options, max_rows=-1):
         """
@@ -105,10 +93,8 @@ class ExcelConnector(SourceConnection):
         if not self._wb:
             return iter([]), []
         if table not in self._wb:
-            # table not found
             return iter([]), []
         worksheet = self._wb[table]
-        # get options
         has_header = options.get("header", False)
         skip_rows = options.get("row", 0)
         skip_columns = options.get("column", 0)
@@ -133,13 +119,22 @@ class ExcelConnector(SourceConnection):
         else:
             header = []
             rows = chain((first_row,), rows)
-        # iterator for selected columns and skipped rows
         data_iterator = (list(cell.value for cell in islice(row, skip_columns, read_to_col)) for row in rows)
         if stop_at_empty_row:
-            # add condition to iterator
             data_iterator = takewhile(any, data_iterator)
         return data_iterator, header
 
+    def get_table_cell(self, table: str, row: int, column: int, options: dict) -> Any:
+        """See base class."""
+        if not self._wb:
+            raise ReaderError("no connection to workbook")
+        if table not in self._wb:
+            raise ReaderError(f"no sheet called '{table}'")
+        worksheet = self._wb[table]
+        real_row = row + 1 + options.get("row", 0) + (1 if options.get("header", False) else 0)
+        real_column = column + 1 + options.get("column", 0)
+        return worksheet.cell(real_row, real_column).value
+
 
 def get_mapped_data_from_xlsx(filepath):
     """Returns mapped data from given Excel file assuming it has the default Spine Excel format.
@@ -147,7 +142,7 @@ def get_mapped_data_from_xlsx(filepath):
     Args:
         filepath (str): path to Excel file
     """
-    connector = ExcelConnector(None)
+    connector = ExcelReader(None)
     connector.connect_to_source(filepath)
     mapping = connector.create_default_mapping()
     table_mappings = {
@@ -241,19 +236,16 @@ def _get_header(ws, header_row, index_dim_count):
 
 def _create_entity_mappings(metadata, header, index_dim_count):
     class_name = metadata["class_name"]
-    entity_type = metadata["entity_type"]
+    entity_dim_count = int(metadata["entity_dim_count"])
     map_dict = {"name": class_name}
     ent_alt_map_type = "row" if index_dim_count else "column"
-    if entity_type == "object":
+    if entity_dim_count == 0:
         map_dict["map_type"] = "ObjectClass"
         map_dict["objects"] = {"map_type": ent_alt_map_type, "reference": 0}
-    elif entity_type == "relationship":
-        entity_dim_count = int(metadata["entity_dim_count"])
+    else:
         map_dict["map_type"] = "RelationshipClass"
         map_dict["object_classes"] = header[:entity_dim_count]
         map_dict["objects"] = [{"map_type": ent_alt_map_type, "reference": i} for i in range(entity_dim_count)]
-    else:
-        return None, None
     value_type = metadata.get("value_type")
     if value_type is not None:
         value = {"value_type": value_type}
