@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -9,23 +10,20 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains JSONConnector class.
-
-:author: M. Marin (KTH)
-:date:   10.2.2020
-"""
-
-import sys
+""" Contains JSONReader class. """
+import itertools
 import os
+import sys
 import ijson
-from .reader import SourceConnection
+from ijson import IncompleteJSONError
+from ijson.backends.python import UnexpectedSymbol
+from ...exception import ReaderError
+from .reader import Reader, TableProperties
 
 
-class JSONConnector(SourceConnection):
-    """Template class to read data from another QThread."""
+class JSONReader(Reader):
+    """A reader for JSON files."""
 
-    # name of data source, ex: "Text/CSV"
     DISPLAY_NAME = "JSON"
 
     # dict with option specification for source.
@@ -39,44 +37,49 @@ class JSONConnector(SourceConnection):
         self._filename = None
         self._root_prefix = None
 
-    def connect_to_source(self, source):
+    def connect_to_source(self, source, **extras):
         """saves filepath
 
-        Arguments:
-            source {str} -- filepath
+        Args:
+            source (str): filepath
+            **extras: ignored
         """
         self._filename = source
         self._root_prefix = os.path.splitext(os.path.basename(source))[0]
 
     def disconnect(self):
-        """Disconnect from connected source.
-        """
+        """Disconnect from connected source."""
 
-    def get_tables(self):
-        prefixes = dict()
-        with open(self._filename) as f:
+    def get_tables_and_properties(self):
+        prefixes = {}
+        with open(self._filename, encoding="utf-8") as f:
             for prefix, event, _ in ijson.parse(f):
                 if event in ("start_map", "start_array"):
                     prefixes[".".join([self._root_prefix, prefix])] = None
-        return [self._root_prefix] + list(prefixes.keys())[1:]
+        tables = {prefix: TableProperties() for prefix in itertools.islice(prefixes.keys(), 1, None)}
+        tables[self._root_prefix] = TableProperties()
+        return tables
 
     def file_iterator(self, table, options, max_rows=-1):
         if max_rows == -1:
             max_rows = sys.maxsize
-        max_depth = options["max_depth"]
+        max_depth = options.get("max_depth", self.OPTIONS["max_depth"]["default"])
         prefix = ".".join(table.split(".")[1:])
-        with open(self._filename) as f:
+        with open(self._filename, "rb") as f:
             row = 0
-            for obj in ijson.items(f, prefix):
-                for x in _tabulize_json(obj):
-                    if row > max_rows:
-                        return
-                    yield x[:max_depth]
-                    row += 1
+            try:
+                for obj in ijson.items(f, prefix):
+                    for x in _tabulize_json(obj):
+                        if row > max_rows:
+                            return
+                        yield x[:max_depth]
+                        row += 1
+            except (IncompleteJSONError, UnexpectedSymbol) as error:
+                raise ReaderError(f"failed to read JSON: {error}") from error
 
     def get_data_iterator(self, table, options, max_rows=-1):
         """
-        Return data read from data source table in table. If max_rows is
+        Returns data read from data source table in table. If max_rows is
         specified only that number of rows.
         """
         return self.file_iterator(table, options, max_rows=max_rows), []
