@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Database API contributors
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -9,37 +10,35 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Tests for the parameter_value module.
-
-:authors: A. Soininen (VTT)
-:date:   7.6.2019
-"""
-
+""" Tests for the parameter_value module. """
 from datetime import datetime
 import json
+import pickle
 import unittest
 import dateutil.parser
 from dateutil.relativedelta import relativedelta
 import numpy as np
 import numpy.testing
 from spinedb_api.parameter_value import (
-    convert_containers_to_maps,
-    convert_leaf_maps_to_specialized_containers,
-    convert_map_to_table,
-    duration_to_relativedelta,
-    relativedelta_to_duration,
-    from_database,
-    to_database,
     Array,
     DateTime,
     Duration,
-    IndexedNumberArray,
     Map,
     TimePattern,
+    TimeSeries,
     TimeSeriesFixedResolution,
     TimeSeriesVariableResolution,
-    TimeSeries,
+    convert_containers_to_maps,
+    convert_leaf_maps_to_specialized_containers,
+    convert_map_to_table,
+    deep_copy_value,
+    duration_to_relativedelta,
+    fancy_type_to_type_and_rank,
+    from_database,
+    relativedelta_to_duration,
+    to_database,
+    type_and_rank_to_fancy_type,
+    type_for_value,
 )
 
 
@@ -145,13 +144,13 @@ class TestParameterValue(unittest.TestCase):
 
     def test_from_database_plain_number(self):
         database_value = b"23.0"
-        value = from_database(database_value, value_type=None)
+        value = from_database(database_value, type_="float")
         self.assertTrue(isinstance(value, float))
         self.assertEqual(value, 23.0)
 
     def test_from_database_boolean(self):
         database_value = b"true"
-        value = from_database(database_value, value_type=None)
+        value = from_database(database_value, type_="boolean")
         self.assertTrue(isinstance(value, bool))
         self.assertEqual(value, True)
 
@@ -160,7 +159,7 @@ class TestParameterValue(unittest.TestCase):
         database_value, value_type = to_database(value)
         value_as_float = json.loads(database_value)
         self.assertEqual(value_as_float, value)
-        self.assertIsNone(value_type)
+        self.assertEqual(value_type, "float")
 
     def test_to_database_DateTime(self):
         value = DateTime(datetime(year=2019, month=6, day=26, hour=12, minute=50, second=13))
@@ -171,7 +170,7 @@ class TestParameterValue(unittest.TestCase):
 
     def test_from_database_DateTime(self):
         database_value = b'{"data": "2019-06-01T22:15:00+01:00"}'
-        value = from_database(database_value, value_type="date_time")
+        value = from_database(database_value, type_="date_time")
         self.assertEqual(value.value, dateutil.parser.parse("2019-06-01T22:15:00+01:00"))
 
     def test_DateTime_to_database(self):
@@ -183,17 +182,17 @@ class TestParameterValue(unittest.TestCase):
 
     def test_from_database_Duration(self):
         database_value = b'{"data": "4 seconds"}'
-        value = from_database(database_value, value_type="duration")
+        value = from_database(database_value, type_="duration")
         self.assertEqual(value.value, relativedelta(seconds=4))
 
     def test_from_database_Duration_default_units(self):
         database_value = b'{"data": 23}'
-        value = from_database(database_value, value_type="duration")
+        value = from_database(database_value, type_="duration")
         self.assertEqual(value.value, relativedelta(minutes=23))
 
     def test_from_database_Duration_legacy_list_format_converted_to_Array(self):
         database_value = b'{"data": ["1 hour", "1h", 60, "2 hours"]}'
-        value = from_database(database_value, value_type="duration")
+        value = from_database(database_value, type_="duration")
         expected = Array([Duration("1h"), Duration("1h"), Duration("1h"), Duration("2h")])
         self.assertEqual(value, expected)
 
@@ -213,7 +212,7 @@ class TestParameterValue(unittest.TestCase):
           }
         }
         """
-        value = from_database(database_value, value_type="time_pattern")
+        value = from_database(database_value, type_="time_pattern")
         self.assertEqual(len(value), 2)
         self.assertEqual(value.indexes, ["m1-4,m9-12", "m5-8"])
         numpy.testing.assert_equal(value.values, numpy.array([300.0, 221.5]))
@@ -228,7 +227,7 @@ class TestParameterValue(unittest.TestCase):
           }
         }
         """
-        value = from_database(database_value, value_type="time_pattern")
+        value = from_database(database_value, type_="time_pattern")
         self.assertEqual(value.indexes, ["M1-12"])
         numpy.testing.assert_equal(value.values, numpy.array([300.0]))
         self.assertEqual(value.index_name, "index")
@@ -269,7 +268,7 @@ class TestParameterValue(unittest.TestCase):
                               "1983-05-25": 6
                           }
                       }"""
-        time_series = from_database(releases, value_type="time_series")
+        time_series = from_database(releases, type_="time_series")
         self.assertEqual(
             time_series.indexes,
             numpy.array(
@@ -290,7 +289,7 @@ class TestParameterValue(unittest.TestCase):
                           },
                           "index_name": "index"
                       }"""
-        time_series = from_database(releases, value_type="time_series")
+        time_series = from_database(releases, type_="time_series")
         self.assertEqual(time_series.index_name, "index")
 
     def test_from_database_TimeSeriesVariableResolution_as_two_column_array(self):
@@ -301,7 +300,7 @@ class TestParameterValue(unittest.TestCase):
                               ["1983-05-25", 6]
                           ]
                       }"""
-        time_series = from_database(releases, value_type="time_series")
+        time_series = from_database(releases, type_="time_series")
         self.assertEqual(
             time_series.indexes,
             numpy.array(
@@ -322,7 +321,7 @@ class TestParameterValue(unittest.TestCase):
                           ],
                           "index_name": "index"
                       }"""
-        time_series = from_database(releases, value_type="time_series")
+        time_series = from_database(releases, type_="time_series")
         self.assertEqual(time_series.index_name, "index")
 
     def test_from_database_TimeSeriesFixedResolution_default_repeat(self):
@@ -333,7 +332,7 @@ class TestParameterValue(unittest.TestCase):
                                    "data": [["2019-07-02T10:00:00", 7.0],
                                             ["2019-07-02T10:00:01", 4.0]]
                                }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertTrue(time_series.ignore_year)
         self.assertFalse(time_series.repeat)
 
@@ -380,7 +379,7 @@ class TestParameterValue(unittest.TestCase):
                                    },
                                    "data": [7.0, 5.0, 8.1]
                                }"""
-        time_series = from_database(days_of_our_lives, value_type="time_series")
+        time_series = from_database(days_of_our_lives, type_="time_series")
         self.assertEqual(len(time_series), 3)
         self.assertEqual(
             time_series.indexes,
@@ -403,7 +402,7 @@ class TestParameterValue(unittest.TestCase):
                                 "data": [1, 2, 3, 4, 5, 8]
                             }
         """
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertEqual(len(time_series), 6)
         self.assertEqual(
             time_series.indexes,
@@ -432,7 +431,7 @@ class TestParameterValue(unittest.TestCase):
                                 "index_name": "index"
                             }
         """
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertEqual(time_series.index_name, "index")
 
     def test_from_database_TimeSeriesFixedResolution_resolution_list(self):
@@ -445,7 +444,7 @@ class TestParameterValue(unittest.TestCase):
                                 },
                                 "data": [7.0, 5.0, 8.1, -4.1]
                             }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertEqual(len(time_series), 4)
         self.assertEqual(
             time_series.indexes,
@@ -475,7 +474,7 @@ class TestParameterValue(unittest.TestCase):
                                    },
                                    "data": [7.0, 5.0, 8.1]
                                }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertEqual(len(time_series), 3)
         self.assertEqual(len(time_series.resolution), 1)
         self.assertEqual(time_series.resolution[0], relativedelta(hours=1))
@@ -488,7 +487,7 @@ class TestParameterValue(unittest.TestCase):
                                    },
                                    "data": [7.0, 5.0, 8.1]
                                }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertEqual(len(time_series), 3)
         self.assertEqual(len(time_series.resolution), 1)
         self.assertEqual(time_series.resolution[0], relativedelta(minutes=30))
@@ -499,7 +498,7 @@ class TestParameterValue(unittest.TestCase):
                                    },
                                    "data": [7.0, 5.0, 8.1]
                                }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertEqual(len(time_series), 3)
         self.assertEqual(len(time_series.resolution), 2)
         self.assertEqual(time_series.resolution[0], relativedelta(minutes=30))
@@ -515,7 +514,7 @@ class TestParameterValue(unittest.TestCase):
                                    },
                                    "data": [7.0, 5.0, 8.1]
                                }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertFalse(time_series.ignore_year)
         # Should be true if start is omitted
         database_value = b"""{
@@ -525,7 +524,7 @@ class TestParameterValue(unittest.TestCase):
                                    },
                                    "data": [7.0, 5.0, 8.1]
                                }"""
-        time_series = from_database(database_value, value_type="time_series")
+        time_series = from_database(database_value, type_="time_series")
         self.assertTrue(time_series.ignore_year)
 
     def test_TimeSeriesFixedResolution_to_database(self):
@@ -604,7 +603,7 @@ class TestParameterValue(unittest.TestCase):
 
     def test_from_database_Map_with_index_name(self):
         database_value = b'{"index_type":"str", "index_name": "index", "data":[["a", 1.1]]}'
-        value = from_database(database_value, value_type="map")
+        value = from_database(database_value, type_="map")
         self.assertIsInstance(value, Map)
         self.assertEqual(value.indexes, ["a"])
         self.assertEqual(value.values, [1.1])
@@ -612,7 +611,7 @@ class TestParameterValue(unittest.TestCase):
 
     def test_from_database_Map_dictionary_format(self):
         database_value = b'{"index_type":"str", "data":{"a": 1.1, "b": 2.2}}'
-        value = from_database(database_value, value_type="map")
+        value = from_database(database_value, type_="map")
         self.assertIsInstance(value, Map)
         self.assertEqual(value.indexes, ["a", "b"])
         self.assertEqual(value.values, [1.1, 2.2])
@@ -620,21 +619,21 @@ class TestParameterValue(unittest.TestCase):
 
     def test_from_database_Map_two_column_array_format(self):
         database_value = b'{"index_type":"float", "data":[[1.1, "a"], [2.2, "b"]]}'
-        value = from_database(database_value, value_type="map")
+        value = from_database(database_value, type_="map")
         self.assertIsInstance(value, Map)
         self.assertEqual(value.indexes, [1.1, 2.2])
         self.assertEqual(value.values, ["a", "b"])
         self.assertEqual(value.index_name, "x")
 
     def test_from_database_Map_nested_maps(self):
-        database_value = b'''
+        database_value = b"""
         {
              "index_type": "duration",
               "data":[["1 hour", {"type": "map",
                                   "index_type": "date_time",
                                   "data": {"2020-01-01T12:00": {"type":"duration", "data":"3 hours"}}}]]
-        }'''
-        value = from_database(database_value, value_type="map")
+        }"""
+        value = from_database(database_value, type_="map")
         self.assertEqual(value.indexes, [Duration("1 hour")])
         nested_map = value.values[0]
         self.assertIsInstance(nested_map, Map)
@@ -642,15 +641,15 @@ class TestParameterValue(unittest.TestCase):
         self.assertEqual(nested_map.values, [Duration("3 hours")])
 
     def test_from_database_Map_with_TimeSeries_values(self):
-        database_value = b'''
+        database_value = b"""
         {
              "index_type": "duration",
               "data":[["1 hour", {"type": "time_series",
                                   "data": [["2020-01-01T12:00", -3.0], ["2020-01-02T12:00", -9.3]]
                                  }
                      ]]
-        }'''
-        value = from_database(database_value, value_type="map")
+        }"""
+        value = from_database(database_value, type_="map")
         self.assertEqual(value.indexes, [Duration("1 hour")])
         self.assertEqual(
             value.values,
@@ -658,22 +657,22 @@ class TestParameterValue(unittest.TestCase):
         )
 
     def test_from_database_Map_with_Array_values(self):
-        database_value = b'''
+        database_value = b"""
         {
              "index_type": "duration",
               "data":[["1 hour", {"type": "array", "data": [-3.0, -9.3]}]]
-        }'''
-        value = from_database(database_value, value_type="map")
+        }"""
+        value = from_database(database_value, type_="map")
         self.assertEqual(value.indexes, [Duration("1 hour")])
         self.assertEqual(value.values, [Array([-3.0, -9.3])])
 
     def test_from_database_Map_with_TimePattern_values(self):
-        database_value = b'''
+        database_value = b"""
         {
              "index_type": "float",
               "data":[["2.3", {"type": "time_pattern", "data": {"M1-2": -9.3, "M3-12": -3.9}}]]
-        }'''
-        value = from_database(database_value, value_type="map")
+        }"""
+        value = from_database(database_value, type_="map")
         self.assertEqual(value.indexes, [2.3])
         self.assertEqual(value.values, [TimePattern(["M1-2", "M3-12"], [-9.3, -3.9])])
 
@@ -681,7 +680,7 @@ class TestParameterValue(unittest.TestCase):
         map_value = Map(["a", "b"], [1.1, 2.2])
         db_value, value_type = to_database(map_value)
         raw = json.loads(db_value)
-        self.assertEqual(raw, {"index_type": "str", "data": [["a", 1.1], ["b", 2.2]]})
+        self.assertEqual(raw, {"index_type": "str", "rank": 1, "data": [["a", 1.1], ["b", 2.2]]})
         self.assertEqual(value_type, "map")
 
     def test_Map_to_database_with_index_names(self):
@@ -696,8 +695,18 @@ class TestParameterValue(unittest.TestCase):
             {
                 "index_type": "str",
                 "index_name": "index",
+                "rank": 2,
                 "data": [
-                    ["A", {"type": "map", "index_type": "str", "index_name": "nested index", "data": [["a", 0.3]]}]
+                    [
+                        "A",
+                        {
+                            "type": "map",
+                            "index_type": "str",
+                            "index_name": "nested index",
+                            "rank": 1,
+                            "data": [["a", 0.3]],
+                        },
+                    ]
                 ],
             },
         )
@@ -713,6 +722,7 @@ class TestParameterValue(unittest.TestCase):
         raw = json.loads(db_value)
         expected = {
             "index_type": "str",
+            "rank": 2,
             "data": [
                 ["a", {"type": "time_series", "data": {"2020-01-01T12:00:00": 2.3, "2020-01-02T12:00:00": 4.5}}],
                 ["b", {"type": "time_series", "data": {"2020-01-01T12:00:00": -4.5, "2020-01-02T12:00:00": -2.3}}],
@@ -730,10 +740,16 @@ class TestParameterValue(unittest.TestCase):
             raw,
             {
                 "index_type": "date_time",
+                "rank": 2,
                 "data": [
                     [
                         "2020-01-01T13:00:00",
-                        {"type": "map", "index_type": "duration", "data": [["2M", {"type": "duration", "data": "5D"}]]},
+                        {
+                            "type": "map",
+                            "index_type": "duration",
+                            "rank": 1,
+                            "data": [["2M", {"type": "duration", "data": "5D"}]],
+                        },
                     ]
                 ],
             },
@@ -773,7 +789,7 @@ class TestParameterValue(unittest.TestCase):
             "value_type": "float",
             "data": [1.2, 2.3]
         }"""
-        array = from_database(database_value, value_type="array")
+        array = from_database(database_value, type_="array")
         self.assertEqual(array.values, [1.2, 2.3])
         self.assertEqual(array.indexes, [0, 1])
         self.assertEqual(array.index_name, "i")
@@ -782,7 +798,7 @@ class TestParameterValue(unittest.TestCase):
         database_value = b"""{
             "data": [1.2, 2.3]
         }"""
-        array = from_database(database_value, value_type="array")
+        array = from_database(database_value, type_="array")
         self.assertEqual(array.values, [1.2, 2.3])
         self.assertEqual(array.indexes, [0, 1])
         self.assertEqual(array.index_name, "i")
@@ -792,7 +808,7 @@ class TestParameterValue(unittest.TestCase):
             "value_type": "str",
             "data": ["A", "B"]
         }"""
-        array = from_database(database_value, value_type="array")
+        array = from_database(database_value, type_="array")
         self.assertEqual(array.values, ["A", "B"])
         self.assertEqual(array.indexes, [0, 1])
         self.assertEqual(array.index_name, "i")
@@ -802,7 +818,7 @@ class TestParameterValue(unittest.TestCase):
             "value_type": "date_time",
             "data": ["2020-03-25T10:34:00"]
         }"""
-        array = from_database(database_value, value_type="array")
+        array = from_database(database_value, type_="array")
         self.assertEqual(array.values, [DateTime("2020-03-25T10:34:00")])
         self.assertEqual(array.indexes, [0])
         self.assertEqual(array.index_name, "i")
@@ -812,7 +828,7 @@ class TestParameterValue(unittest.TestCase):
             "value_type": "duration",
             "data": ["2 years", "7 seconds"]
         }"""
-        array = from_database(database_value, value_type="array")
+        array = from_database(database_value, type_="array")
         self.assertEqual(array.values, [Duration("2 years"), Duration("7s")])
         self.assertEqual(array.indexes, [0, 1])
         self.assertEqual(array.index_name, "i")
@@ -823,7 +839,7 @@ class TestParameterValue(unittest.TestCase):
             "index_name": "index",
             "data": [1.2]
         }"""
-        array = from_database(database_value, value_type="array")
+        array = from_database(database_value, type_="array")
         self.assertEqual(array.values, [1.2])
         self.assertEqual(array.indexes, [0])
         self.assertEqual(array.index_name, "index")
@@ -866,6 +882,13 @@ class TestParameterValue(unittest.TestCase):
         map_value = Map(["A"], [nested_map])
         self.assertEqual(map_value, Map(["A"], [Map(["a"], [-2.3])]))
 
+    def test_Map_inequality(self):
+        map_value = Map(["1", "2", "3", "4", "5"], [-2.3, 2.3, 2.3, 2.3, 2.3])
+        self.assertNotEqual(map_value, Map(["a", "b"], [2.3, -2.3]))
+        self.assertNotEqual(
+            Map([DateTime("2024-05-30T00:00:00")], [2.3]), Map([DateTime("2024-05-30T10:00:00")], [2.3])
+        )
+
     def test_TimePattern_equality(self):
         pattern = TimePattern(["D1-2", "D3-7"], np.array([-2.3, -5.0]))
         self.assertEqual(pattern, pattern)
@@ -890,11 +913,11 @@ class TestParameterValue(unittest.TestCase):
         inequal_series = TimeSeriesVariableResolution(["2000-01-01T00:00", "2002-01-01T00:00"], [4.2, 2.4], False, True)
         self.assertNotEqual(series, inequal_series)
 
-    def test_IndexedValue_constructor_converts_values_to_floats(self):
-        value = IndexedNumberArray("", [4, -9, 11])
+    def test_TimeSeries_constructor_converts_values_to_floats(self):
+        value = TimeSeries([4, -9, 11], False, False)
         self.assertEqual(value.values.dtype, np.dtype(float))
         numpy.testing.assert_equal(value.values, numpy.array([4.0, -9.0, 11.0]))
-        value = IndexedNumberArray("", numpy.array([16, -251, 99]))
+        value = TimeSeries(numpy.array([16, -251, 99]), False, False)
         self.assertEqual(value.values.dtype, np.dtype(float))
         numpy.testing.assert_equal(value.values, numpy.array([16.0, -251.0, 99.0]))
 
@@ -1001,6 +1024,196 @@ class TestParameterValue(unittest.TestCase):
         nested_map = Map(["A", "B"], [map1, map2])
         self.assertEqual(nested_map, {"A": {"a": -3.2, "b": -2.3}, "B": {"c": 3.2, "d": 2.3}})
 
+    def test_deep_copy_value_for_scalars(self):
+        x = None
+        copy_of_x = deep_copy_value(x)
+        self.assertIsNone(copy_of_x)
+        x = 1.0
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        x = "y"
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        x = Duration("3h")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+        x = DateTime("2024-03-25T15:58:33")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+
+    def test_deep_copy_for_arrays(self):
+        x = Array([], value_type=float, index_name="floaters")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+        x = Array(["1", "2", "3"], index_name="number likes")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+
+    def test_deep_copy_time_pattern(self):
+        x = TimePattern(["M1-12"], [2.3], index_name="moments")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+
+    def test_deep_copy_time_series_fixed_resolution(self):
+        x = TimeSeriesFixedResolution(
+            "2024-03-25T16:08:23", "4h", [2.3, 23.0, 5.0], ignore_year=True, repeat=True, index_name="my times"
+        )
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+
+    def test_deep_copy_time_series_variable_resolution(self):
+        x = TimeSeriesVariableResolution(
+            ["2024-03-25T16:10:23", "2024-04-26T16:11:23"],
+            [2.3, 23.0],
+            ignore_year=True,
+            repeat=True,
+            index_name="your times",
+        )
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+
+    def test_deep_copy_map(self):
+        x = Map([], [], index_type=str, index_name="first i")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+        x = Map(["T1", "T2"], [2.3, 23.0], index_name="our times")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+        leaf = Map(["t1"], [2.3], index_name="inner")
+        x = Map(["T1"], [leaf], index_name="outer")
+        copy_of_x = deep_copy_value(x)
+        self.assertEqual(x, copy_of_x)
+        self.assertIsNot(x, copy_of_x)
+        self.assertIsNot(x.get_value("T1"), copy_of_x.get_value("T1"))
+
+
+class TestTimeSeriesVariableResolution:
+    def test_get_value(self):
+        time_series = TimeSeriesVariableResolution(
+            ["2025-07-01T15:45", "2025-07-01T16:45"], [2.3, 3.2], ignore_year=False, repeat=False, index_name="y"
+        )
+        assert time_series.get_value(np.datetime64("2025-07-01T15:45")) == 2.3
+        assert time_series.get_value(np.datetime64("2025-07-01T16:45")) == 3.2
+        assert time_series.get_value(np.datetime64("2025-07-01T16:00")) is None
+
+    def test_set_value(self):
+        time_series = TimeSeriesVariableResolution(
+            ["2025-07-01T15:45", "2025-07-01T16:45"], [2.3, 3.2], ignore_year=False, repeat=False, index_name="y"
+        )
+        time_series.set_value(np.datetime64("2025-07-01T15:45"), -2.3)
+        assert time_series.get_value(np.datetime64("2025-07-01T15:45")) == -2.3
+        time_series.set_value(np.datetime64("2025-07-01T16:45"), -3.2)
+        assert time_series.get_value(np.datetime64("2025-07-01T16:45")) == -3.2
+
+
+class TestTimeSeriesFixedResolution:
+    def test_get_value(self):
+        time_series = TimeSeriesFixedResolution(
+            "2025-07-01T15:45", "1h", [2.3, 3.2], ignore_year=False, repeat=False, index_name="y"
+        )
+        assert time_series.get_value(np.datetime64("2025-07-01T15:45")) == 2.3
+        assert time_series.get_value(np.datetime64("2025-07-01T16:45")) == 3.2
+        assert time_series.get_value(np.datetime64("2025-07-01T16:00")) is None
+
+    def test_get_value_with_multiresolution(self):
+        time_series = TimeSeriesFixedResolution(
+            "2025-07-01T15:45", ["1h", "15m"], [2.3, 3.2, -2.3, -3.2], ignore_year=False, repeat=False, index_name="y"
+        )
+        assert time_series.get_value(np.datetime64("2025-07-01T15:45")) == 2.3
+        assert time_series.get_value(np.datetime64("2025-07-01T16:45")) == 3.2
+        assert time_series.get_value(np.datetime64("2025-07-01T17:00")) == -2.3
+        assert time_series.get_value(np.datetime64("2025-07-01T18:00")) == -3.2
+        assert time_series.get_value(np.datetime64("2025-07-01T16:00")) is None
+
+    def test_set_value(self):
+        time_series = TimeSeriesFixedResolution(
+            "2025-07-01T15:45", "1h", [2.3, 3.2], ignore_year=False, repeat=False, index_name="y"
+        )
+        time_series.set_value(np.datetime64("2025-07-01T15:45"), -2.3)
+        assert time_series.get_value(np.datetime64("2025-07-01T15:45")) == -2.3
+        time_series.set_value(np.datetime64("2025-07-01T16:45"), -3.2)
+        assert time_series.get_value(np.datetime64("2025-07-01T16:45")) == -3.2
+
+    def test_set_value_with_multiresolution(self):
+        time_series = TimeSeriesFixedResolution(
+            "2025-07-01T15:45", ["1h", "15m"], [2.3, 3.2, -2.3, -3.2], ignore_year=False, repeat=False, index_name="y"
+        )
+        time_series.set_value(np.datetime64("2025-07-01T15:45"), -1.1)
+        assert time_series.get_value(np.datetime64("2025-07-01T15:45")) == -1.1
+        time_series.set_value(np.datetime64("2025-07-01T16:45"), -2.2)
+        assert time_series.get_value(np.datetime64("2025-07-01T16:45")) == -2.2
+        time_series.set_value(np.datetime64("2025-07-01T17:00"), -3.3)
+        assert time_series.get_value(np.datetime64("2025-07-01T17:00")) == -3.3
+        time_series.set_value(np.datetime64("2025-07-01T18:00"), -4.4)
+        assert time_series.get_value(np.datetime64("2025-07-01T18:00")) == -4.4
+
+
+class TestPickling(unittest.TestCase):
+    def test_array_is_picklable(self):
+        x = Array([2.3])
+        deserialized = pickle.loads(pickle.dumps(x))
+        self.assertEqual(deserialized, x)
+        x = Array([DateTime("2024-07-30T09:00:00")])
+        deserialized = pickle.loads(pickle.dumps(x))
+        self.assertEqual(deserialized, x)
+
+    def test_time_pattern_is_picklable(self):
+        x = TimePattern(["D1-7"], [2.3])
+        deserialized = pickle.loads(pickle.dumps(x))
+        self.assertEqual(deserialized, x)
+
+
+class TestTypeAndRankToFancyType(unittest.TestCase):
+    def test_function_works(self):
+        self.assertEqual(type_and_rank_to_fancy_type("float", 0), "float")
+        self.assertEqual(type_and_rank_to_fancy_type("str", 0), "str")
+        self.assertEqual(type_and_rank_to_fancy_type("bool", 0), "bool")
+        self.assertEqual(type_and_rank_to_fancy_type("date_time", 0), "date_time")
+        self.assertEqual(type_and_rank_to_fancy_type("duration", 0), "duration")
+        self.assertEqual(type_and_rank_to_fancy_type("array", 1), "array")
+        self.assertEqual(type_and_rank_to_fancy_type("time_pattern", 1), "time_pattern")
+        self.assertEqual(type_and_rank_to_fancy_type("time_series", 1), "time_series")
+        self.assertEqual(type_and_rank_to_fancy_type("map", 23), "23d_map")
+
+
+class TestFancyTypeToTypeAndRank(unittest.TestCase):
+    def test_function_works(self):
+        self.assertEqual(fancy_type_to_type_and_rank("float"), ("float", 0))
+        self.assertEqual(fancy_type_to_type_and_rank("str"), ("str", 0))
+        self.assertEqual(fancy_type_to_type_and_rank("bool"), ("bool", 0))
+        self.assertEqual(fancy_type_to_type_and_rank("date_time"), ("date_time", 0))
+        self.assertEqual(fancy_type_to_type_and_rank("duration"), ("duration", 0))
+        self.assertEqual(fancy_type_to_type_and_rank("array"), ("array", 1))
+        self.assertEqual(fancy_type_to_type_and_rank("time_pattern"), ("time_pattern", 1))
+        self.assertEqual(fancy_type_to_type_and_rank("time_series"), ("time_series", 1))
+        self.assertEqual(fancy_type_to_type_and_rank("23d_map"), ("map", 23))
+
+
+class TestTypeForValue(unittest.TestCase):
+    def test_function_works(self):
+        self.assertEqual(type_for_value(None), (None, 0))
+        self.assertEqual(type_for_value(2.3), ("float", 0))
+        self.assertEqual(type_for_value("debug"), ("str", 0))
+        self.assertEqual(type_for_value(False), ("bool", 0))
+        self.assertEqual(type_for_value(DateTime("2024-07-25T11:00:00")), ("date_time", 0))
+        self.assertEqual(type_for_value(Duration("23D")), ("duration", 0))
+        self.assertEqual(type_for_value(Array(["a", "b"])), ("array", 1))
+        self.assertEqual(type_for_value(TimePattern(["D1-7"], [8.0])), ("time_pattern", 1))
+        self.assertEqual(
+            type_for_value(TimeSeriesVariableResolution(["2024-07-24T11:00:00"], [2.3], False, False)),
+            ("time_series", 1),
+        )
+        self.assertEqual(type_for_value(Map(["a", "b"], [Map(["i"], [2.3]), 23.0])), ("map", 2))
+
 
 if __name__ == "__main__":
     unittest.main()
